@@ -1,21 +1,47 @@
-#include <QtGui>
-#include <QtXml>
+/******************************************************************************
+ *
+ * Copyright (C) 1997-2019 by Dimitri van Heesch.
+ *
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
+ * for any purpose. It is provided "as is" without express or implied warranty.
+ * See the GNU General Public License for more details.
+ *
+ */
+
 #include "expert.h"
 #include "inputbool.h"
 #include "inputstring.h"
 #include "inputint.h"
 #include "inputstring.h"
 #include "inputstrlist.h"
+#include "inputobsolete.h"
 #include "config.h"
 #include "version.h"
 #include "configdoc.h"
 #include "settings.h"
 
-#define SA(x) QString::fromAscii(x)
+#include <QTreeWidget>
+#include <QStackedWidget>
+#include <QTextBrowser>
+#include <QSplitter>
+#include <QGridLayout>
+#include <QPushButton>
+#include <QScrollArea>
+#include <QFile>
+#include <QMessageBox>
+#include <QSettings>
+#include <QTextStream>
+#include <QFileInfo>
+#include <QRegularExpression>
+
+#define SA(x) QString::fromLatin1(x)
+
 
 static QString convertToComment(const QString &s)
 {
-  if (s.isEmpty()) 
+  if (s.isEmpty())
   {
     return QString();
   }
@@ -48,7 +74,7 @@ Expert::Expert()
   m_treeWidget = new QTreeWidget;
   m_treeWidget->setColumnCount(1);
   m_topicStack = new QStackedWidget;
-  m_inShowHelp = FALSE;
+  m_inShowHelp = false;
 
   QFile file(SA(":/config.xml"));
   QString err;
@@ -69,7 +95,7 @@ Expert::Expert()
   createTopics(m_rootElement);
   m_helper = new QTextBrowser;
   m_helper->setReadOnly(true);
-  m_helper->setOpenExternalLinks(TRUE);
+  m_helper->setOpenExternalLinks(true);
   m_splitter = new QSplitter(Qt::Vertical);
   m_splitter->addWidget(m_treeWidget);
   m_splitter->addWidget(m_helper);
@@ -97,7 +123,7 @@ Expert::Expert()
 Expert::~Expert()
 {
   QHashIterator<QString,Input*> i(m_options);
-  while (i.hasNext()) 
+  while (i.hasNext())
   {
     i.next();
     delete i.value();
@@ -114,10 +140,14 @@ void Expert::createTopics(const QDomElement &rootElem)
     {
       // Remove _ from a group name like: Source_Browser
       QString name = childElem.attribute(SA("name")).replace(SA("_"),SA(" "));
-      items.append(new QTreeWidgetItem((QTreeWidget*)0,QStringList(name)));
-      QWidget *widget = createTopicWidget(childElem);
-      m_topics[name] = widget;
-      m_topicStack->addWidget(widget);
+      QString setting = childElem.attribute(SA("setting"));
+      if (setting.isEmpty() || IS_SUPPORTED(setting.toLatin1()))
+      {
+        items.append(new QTreeWidgetItem((QTreeWidget*)nullptr,QStringList(name)));
+        QWidget *widget = createTopicWidget(childElem);
+        m_topics[name] = widget;
+        m_topicStack->addWidget(widget);
+      }
     }
     childElem = childElem.nextSiblingElement();
   }
@@ -193,11 +223,14 @@ static QString getDocsForNode(const QDomElement &child)
       }
       docsVal = docsVal.nextSiblingElement();
     }
-    docs+=SA("<br/>");
-    docs+=SA("<br/>");
-    docs+=SA(" The default value is: <code>")+
-          child.attribute(SA("defval"))+
-          SA("</code>.");
+    if (child.attribute(SA("defval")) != SA(""))
+    {
+      docs+=SA("<br/>");
+      docs+=SA("<br/>");
+      docs+=SA(" The default value is: <code>")+
+            child.attribute(SA("defval"))+
+            SA("</code>.");
+    }
     docs+= SA("<br/>");
   }
   else if (type==SA("int"))
@@ -359,7 +392,7 @@ static QString getDocsForNode(const QDomElement &child)
       }
     }
   }
-  
+
   if (child.hasAttribute(SA("depends")))
   {
     QString dependsOn = child.attribute(SA("depends"));
@@ -373,7 +406,8 @@ static QString getDocsForNode(const QDomElement &child)
 
   // Remove / replace doxygen markup strings
   // the regular expressions are hard to read so the intention will be given
-  QRegExp regexp;
+  // Note: see also configgen.py in the src directory for other doxygen parts
+  QRegularExpression regexp;
   // remove \n at end and replace by a space
   regexp.setPattern(SA("\\n$"));
   docs.replace(regexp,SA(" "));
@@ -397,7 +431,7 @@ static QString getDocsForNode(const QDomElement &child)
   regexp.setPattern(SA("`([^`]+)`"));
   docs.replace(regexp,SA("<code>\\1</code>"));
   // \ref key "desc" -> <code>desc</code>
-  regexp.setPattern(SA("\\\\ref[ ]+[^ ]+[ ]+\"([^ ]+)\""));
+  regexp.setPattern(SA("\\\\ref[ ]+[^ ]+[ ]+\"([^\"]+)\""));
   docs.replace(regexp,SA("<code>\\1</code> "));
   //\ref specials
   // \ref <key> -> description
@@ -407,6 +441,8 @@ static QString getDocsForNode(const QDomElement &child)
   docs.replace(regexp,SA("\"External Indexing and Searching\""));
   regexp.setPattern(SA("\\\\ref[ ]+external"));
   docs.replace(regexp,SA("\"Linking to external documentation\""));
+  regexp.setPattern(SA("\\\\ref[ ]+formulas"));
+  docs.replace(regexp,SA("\"Including formulas\""));
   // fallback for not handled
   docs.replace(SA("\\\\ref"),SA(""));
   // \b word -> <b>word<\b>
@@ -449,6 +485,8 @@ static QString getDocsForNode(const QDomElement &child)
   docs.replace(SA("\\>"),SA("&gt;"));
   regexp.setPattern(SA(" (http:[^ \\)]*)([ \\)])"));
   docs.replace(regexp,SA(" <a href=\"\\1\">\\1</a>\\2"));
+  regexp.setPattern(SA(" (https:[^ \\)]*)([ \\)])"));
+  docs.replace(regexp,SA(" <a href=\"\\1\">\\1</a>\\2"));
   // LaTeX name as formula -> LaTeX
   regexp.setPattern(SA("\\\\f\\$\\\\mbox\\{\\\\LaTeX\\}\\\\f\\$"));
   docs.replace(regexp,SA("LaTeX"));
@@ -471,13 +509,13 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
   while (!child.isNull())
   {
     QString setting = child.attribute(SA("setting"));
-    if (setting.isEmpty() || IS_SUPPORTED(setting.toAscii()))
+    if (setting.isEmpty() || IS_SUPPORTED(setting.toLatin1()))
     {
       QString type = child.attribute(SA("type"));
       QString docs = getDocsForNode(child);
       if (type==SA("bool"))
       {
-        InputBool *boolOption = 
+        InputBool *boolOption =
           new InputBool(
               layout,row,
               child.attribute(SA("id")),
@@ -503,6 +541,10 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
         {
           mode = InputString::StringFile;
         }
+        else if (format==SA("filedir"))
+        {
+          mode = InputString::StringFileDir;
+        }
         else if (format==SA("image"))
         {
           mode = InputString::StringImage;
@@ -511,7 +553,7 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
         {
           mode = InputString::StringFree;
         }
-        InputString *stringOption = 
+        InputString *stringOption =
           new InputString(
               layout,row,
               child.attribute(SA("id")),
@@ -553,7 +595,7 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
       }
       else if (type==SA("int"))
       {
-        InputInt *intOption = 
+        InputInt *intOption =
           new InputInt(
               layout,row,
               child.attribute(SA("id")),
@@ -599,7 +641,7 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
           }
           listVal = listVal.nextSiblingElement();
         }
-        InputStrList *listOption = 
+        InputStrList *listOption =
           new InputStrList(
               layout,row,
               child.attribute(SA("id")),
@@ -616,6 +658,31 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
       }
       else if (type==SA("obsolete"))
       {
+        Input::Kind orgKind;
+        QString orgKindStr = child.attribute(SA("orgtype"));
+        if (orgKindStr==SA("int"))
+        {
+          orgKind = Input::Int;
+        }
+        else if (orgKindStr==SA("bool"))
+        {
+          orgKind = Input::Bool;
+        }
+        else if (orgKindStr==SA("string") || orgKindStr==SA("enum"))
+        {
+          orgKind = Input::String;
+        }
+        else if (orgKindStr==SA("strlist"))
+        {
+          orgKind = Input::StrList;
+        }
+        else
+        {
+          orgKind = Input::Obsolete;
+        }
+        InputObsolete *obsoleteOption =
+          new InputObsolete(child.attribute(SA("id")),orgKind);
+        m_options.insert(child.attribute(SA("id")),obsoleteOption);
         // ignore
       }
       else // should not happen
@@ -633,11 +700,11 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
     QString setting = child.attribute(SA("setting"));
     QString dependsOn = child.attribute(SA("depends"));
     QString id        = child.attribute(SA("id"));
-    if (!dependsOn.isEmpty() && 
-        (setting.isEmpty() || IS_SUPPORTED(setting.toAscii())))
+    if (!dependsOn.isEmpty() &&
+        (setting.isEmpty() || IS_SUPPORTED(setting.toLatin1())))
     {
        Input *parentOption = m_options[dependsOn];
-       if (parentOption==0)
+       if (parentOption==nullptr)
        {
          printf("%s has depends=%s that is not valid\n",
              qPrintable(id),qPrintable(dependsOn));
@@ -658,7 +725,7 @@ QWidget *Expert::createTopicWidget(QDomElement &elem)
 
   // set initial dependencies
   QHashIterator<QString,Input*> i(m_options);
-  while (i.hasNext()) 
+  while (i.hasNext())
   {
     i.next();
     if (i.value())
@@ -682,7 +749,7 @@ void Expert::activateTopic(QTreeWidgetItem *item,QTreeWidgetItem *)
   {
     QWidget *w = m_topics[item->text(0)];
     m_topicStack->setCurrentWidget(w);
-    m_prev->setEnabled(m_topicStack->currentIndex()!=0); 
+    m_prev->setEnabled(m_topicStack->currentIndex()!=0);
     m_next->setEnabled(true);
   }
 }
@@ -690,11 +757,11 @@ void Expert::activateTopic(QTreeWidgetItem *item,QTreeWidgetItem *)
 void Expert::loadSettings(QSettings *s)
 {
   QHashIterator<QString,Input*> i(m_options);
-  while (i.hasNext()) 
+  while (i.hasNext())
   {
     i.next();
     QVariant var = s->value(SA("config/")+i.key());
-    if (i.value())
+    if (i.value() && var.isValid())
     {
       //printf("Loading key %s: type=%d value='%s'\n",qPrintable(i.key()),var.type(),qPrintable(var.toString()));
       i.value()->value() = var;
@@ -706,7 +773,7 @@ void Expert::loadSettings(QSettings *s)
 void Expert::saveSettings(QSettings *s)
 {
   QHashIterator<QString,Input*> i(m_options);
-  while (i.hasNext()) 
+  while (i.hasNext())
   {
     i.next();
     //printf("Saving key %s: type=%d value='%s'\n",qPrintable(i.key()),i.value()->value().type(),qPrintable(i.value()->value().toString()));
@@ -723,16 +790,19 @@ void Expert::loadConfig(const QString &fileName)
   parseConfig(fileName,m_options);
 }
 
-void Expert::saveTopic(QTextStream &t,QDomElement &elem,QTextCodec *codec,
-                       bool brief)
+void Expert::saveTopic(QTextStream &t,QDomElement &elem,TextCodecAdapter *codec,
+                       bool brief,bool condensed,bool convert)
 {
   if (!brief)
   {
-    t << endl;
+    t << "\n";
+  }
+  if (!condensed)
+  {
+    t << "#---------------------------------------------------------------------------\n";
+    t << "# " << elem.attribute(SA("docs")) << "\n";
+    t << "#---------------------------------------------------------------------------\n";
   }
-  t << "#---------------------------------------------------------------------------" << endl;
-  t << "# " << elem.attribute(SA("docs")) << endl;
-  t << "#---------------------------------------------------------------------------" << endl;
   // write options...
   QDomElement childElem = elem.firstChildElement();
   while (!childElem.isNull())
@@ -740,55 +810,56 @@ void Expert::saveTopic(QTextStream &t,QDomElement &elem,QTextCodec *codec,
     QString setting = childElem.attribute(SA("setting"));
     QString type = childElem.attribute(SA("type"));
     QString name = childElem.attribute(SA("id"));
-    if (setting.isEmpty() || IS_SUPPORTED(setting.toAscii()))
+    if (setting.isEmpty() || IS_SUPPORTED(setting.toLatin1()))
     {
       QHash<QString,Input*>::const_iterator i = m_options.find(name);
       if (i!=m_options.end())
       {
         Input *option = i.value();
-        if (option && !brief)
+        if (option->kind()!=Input::Obsolete)
         {
-          t << endl;
-          t << convertToComment(option->templateDocs());
-          t << endl;
+          if (option && !brief)
+          {
+            t << "\n";
+            t << convertToComment(option->templateDocs());
+            t << "\n";
+          }
+          bool toPrint = true;
+          if (option && condensed) toPrint = !option->isDefault();
+          if (toPrint)
+          {
+            t << name.leftJustified(MAX_OPTION_LENGTH) << "=";
+            if (option && !option->isEmpty())
+            {
+              t << " ";
+              option->writeValue(t,codec,convert);
+            }
+            t << "\n";
+          }
         }
-        t << name.leftJustified(MAX_OPTION_LENGTH) << "= ";
-        if (option)
-        {
-          option->writeValue(t,codec);
-        }
-        t << endl;
       }
     }
     childElem = childElem.nextSiblingElement();
   }
 }
 
-bool Expert::writeConfig(QTextStream &t,bool brief)
+bool Expert::writeConfig(QTextStream &t,bool brief, bool condensed, bool convert)
 {
   // write global header
-  t << "# Doxyfile " << versionString << endl << endl; 
-  if (!brief)
+  t << "# Doxyfile " << getDoxygenVersion().c_str() << "\n\n";
+  if (!brief && !condensed)
   {
     t << convertToComment(m_header);
   }
 
-  QTextCodec *codec = 0;
-  Input *option = m_options[QString::fromAscii("DOXYFILE_ENCODING")];
-  if (option)
-  {
-    codec = QTextCodec::codecForName(option->value().toString().toAscii());
-    if (codec==0) // fallback: use UTF-8
-    {
-      codec = QTextCodec::codecForName("UTF-8");
-    }
-  }
+  Input *option = m_options[QString::fromLatin1("DOXYFILE_ENCODING")];
+  TextCodecAdapter codec(option->value().toString().toLatin1());
   QDomElement childElem = m_rootElement.firstChildElement();
   while (!childElem.isNull())
   {
     if (childElem.tagName()==SA("group"))
     {
-      saveTopic(t,childElem,codec,brief);
+      saveTopic(t,childElem,&codec,brief,condensed,convert);
     }
     childElem = childElem.nextSiblingElement();
   }
@@ -809,16 +880,16 @@ void Expert::showHelp(Input *option)
 {
   if (!m_inShowHelp)
   {
-    m_inShowHelp = TRUE;
+    m_inShowHelp = true;
     m_helper->setText(
-        QString::fromAscii("<qt><b>")+option->id()+
-        QString::fromAscii("</b><br>")+
-        QString::fromAscii("<br/>")+
+        QString::fromLatin1("<qt><b>")+option->id()+
+        QString::fromLatin1("</b><br>")+
+        QString::fromLatin1("<br/>")+
         option->docs().
-        replace(QChar::fromAscii('\n'),QChar::fromAscii(' '))+
-        QString::fromAscii("</qt>")
+        replace(QChar::fromLatin1('\n'),QChar::fromLatin1(' '))+
+        QString::fromLatin1("</qt>")
         );
-    m_inShowHelp = FALSE;
+    m_inShowHelp = false;
   }
 }
 
@@ -849,7 +920,7 @@ void Expert::resetToDefaults()
 {
   //printf("Expert::makeDefaults()\n");
   QHashIterator<QString,Input*> i(m_options);
-  while (i.hasNext()) 
+  while (i.hasNext())
   {
     i.next();
     if (i.value())
@@ -862,29 +933,29 @@ void Expert::resetToDefaults()
 static bool stringVariantToBool(const QVariant &v)
 {
   QString s = v.toString().toLower();
-  return s==QString::fromAscii("yes") || s==QString::fromAscii("true") || s==QString::fromAscii("1");
-} 
+  return s==QString::fromLatin1("yes") || s==QString::fromLatin1("true") || s==QString::fromLatin1("1");
+}
 
 static bool getBoolOption(
     const QHash<QString,Input*>&model,const QString &name)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   return stringVariantToBool(option->value());
-} 
+}
 
 static QString getStringOption(
     const QHash<QString,Input*>&model,const QString &name)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   return option->value().toString();
 }
 
 
 bool Expert::htmlOutputPresent(const QString &workingDir) const
 {
-  bool generateHtml = getBoolOption(m_options,QString::fromAscii("GENERATE_HTML"));
+  bool generateHtml = getBoolOption(m_options,QString::fromLatin1("GENERATE_HTML"));
   if (!generateHtml || workingDir.isEmpty()) return false;
   QString indexFile = getHtmlOutputIndex(workingDir);
   QFileInfo fi(indexFile);
@@ -893,8 +964,8 @@ bool Expert::htmlOutputPresent(const QString &workingDir) const
 
 QString Expert::getHtmlOutputIndex(const QString &workingDir) const
 {
-  QString outputDir = getStringOption(m_options,QString::fromAscii("OUTPUT_DIRECTORY"));
-  QString htmlOutputDir = getStringOption(m_options,QString::fromAscii("HTML_OUTPUT"));
+  QString outputDir = getStringOption(m_options,QString::fromLatin1("OUTPUT_DIRECTORY"));
+  QString htmlOutputDir = getStringOption(m_options,QString::fromLatin1("HTML_OUTPUT"));
   //printf("outputDir=%s\n",qPrintable(outputDir));
   //printf("htmlOutputDir=%s\n",qPrintable(htmlOutputDir));
   QString indexFile = workingDir;
@@ -903,8 +974,8 @@ QString Expert::getHtmlOutputIndex(const QString &workingDir) const
     indexFile = outputDir;
   }
   else // append
-  { 
-    indexFile += QString::fromAscii("/")+outputDir;
+  {
+    indexFile += QString::fromLatin1("/")+outputDir;
   }
   if (QFileInfo(htmlOutputDir).isAbsolute()) // override
   {
@@ -912,27 +983,27 @@ QString Expert::getHtmlOutputIndex(const QString &workingDir) const
   }
   else // append
   {
-    indexFile += QString::fromAscii("/")+htmlOutputDir;
+    indexFile += QString::fromLatin1("/")+htmlOutputDir;
   }
-  indexFile+=QString::fromAscii("/index.html");
+  indexFile+=QString::fromLatin1("/index.html");
   return indexFile;
 }
 
 bool Expert::pdfOutputPresent(const QString &workingDir) const
 {
-  bool generateLatex = getBoolOption(m_options,QString::fromAscii("GENERATE_LATEX"));
-  bool pdfLatex = getBoolOption(m_options,QString::fromAscii("USE_PDFLATEX"));
+  bool generateLatex = getBoolOption(m_options,QString::fromLatin1("GENERATE_LATEX"));
+  bool pdfLatex = getBoolOption(m_options,QString::fromLatin1("USE_PDFLATEX"));
   if (!generateLatex || !pdfLatex) return false;
-  QString latexOutput = getStringOption(m_options,QString::fromAscii("LATEX_OUTPUT"));
+  QString latexOutput = getStringOption(m_options,QString::fromLatin1("LATEX_OUTPUT"));
   QString indexFile;
   if (QFileInfo(latexOutput).isAbsolute())
   {
-    indexFile = latexOutput+QString::fromAscii("/refman.pdf");
+    indexFile = latexOutput+QString::fromLatin1("/refman.pdf");
   }
   else
   {
-    indexFile = workingDir+QString::fromAscii("/")+
-                latexOutput+QString::fromAscii("/refman.pdf");
+    indexFile = workingDir+QString::fromLatin1("/")+
+                latexOutput+QString::fromLatin1("/refman.pdf");
   }
   QFileInfo fi(indexFile);
   return fi.exists() && fi.isFile();
