@@ -1,50 +1,82 @@
+/******************************************************************************
+ *
+ * Copyright (C) 1997-2025 by Dimitri van Heesch.
+ *
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
+ * for any purpose. It is provided "as is" without express or implied warranty.
+ * See the GNU General Public License for more details.
+ *
+ */
+
 #include "wizard.h"
 #include "input.h"
 #include "doxywizard.h"
 
 #include <math.h>
-#include <QtGui>
+
+#include <QGridLayout>
+#include <QImage>
+#include <QLabel>
+#include <QHBoxLayout>
+#include <QPushButton>
+#include <QPixmap>
+#include <QPainter>
+#include <QMouseEvent>
+#include <QLineEdit>
+#include <QCheckBox>
+#include <QFileInfo>
+#include <QFileDialog>
+#include <QButtonGroup>
+#include <QGroupBox>
+#include <QRadioButton>
+#include <QTreeWidget>
+#include <QStackedWidget>
+#include <qdrawutil.h>
+
 
 // options configurable via the wizard
-#define STR_PROJECT_NAME          QString::fromAscii("PROJECT_NAME")
-#define STR_PROJECT_LOGO          QString::fromAscii("PROJECT_LOGO")
-#define STR_PROJECT_BRIEF         QString::fromAscii("PROJECT_BRIEF")
-#define STR_INPUT                 QString::fromAscii("INPUT")
-#define STR_OUTPUT_DIRECTORY      QString::fromAscii("OUTPUT_DIRECTORY")
-#define STR_PROJECT_NUMBER        QString::fromAscii("PROJECT_NUMBER")
-#define STR_RECURSIVE             QString::fromAscii("RECURSIVE")
-#define STR_OPTIMIZE_OUTPUT_FOR_C QString::fromAscii("OPTIMIZE_OUTPUT_FOR_C")
-#define STR_OPTIMIZE_OUTPUT_JAVA  QString::fromAscii("OPTIMIZE_OUTPUT_JAVA")
-#define STR_OPTIMIZE_FOR_FORTRAN  QString::fromAscii("OPTIMIZE_FOR_FORTRAN")
-#define STR_OPTIMIZE_OUTPUT_VHDL  QString::fromAscii("OPTIMIZE_OUTPUT_VHDL")
-#define STR_CPP_CLI_SUPPORT       QString::fromAscii("CPP_CLI_SUPPORT")
-#define STR_HIDE_SCOPE_NAMES      QString::fromAscii("HIDE_SCOPE_NAMES")
-#define STR_EXTRACT_ALL           QString::fromAscii("EXTRACT_ALL")
-#define STR_SOURCE_BROWSER        QString::fromAscii("SOURCE_BROWSER")
-#define STR_GENERATE_HTML         QString::fromAscii("GENERATE_HTML")
-#define STR_GENERATE_LATEX        QString::fromAscii("GENERATE_LATEX")
-#define STR_GENERATE_MAN          QString::fromAscii("GENERATE_MAN")
-#define STR_GENERATE_RTF          QString::fromAscii("GENERATE_RTF")
-#define STR_GENERATE_XML          QString::fromAscii("GENERATE_XML")
-#define STR_GENERATE_HTMLHELP     QString::fromAscii("GENERATE_HTMLHELP")
-#define STR_GENERATE_TREEVIEW     QString::fromAscii("GENERATE_TREEVIEW")
-#define STR_USE_PDFLATEX          QString::fromAscii("USE_PDFLATEX")
-#define STR_PDF_HYPERLINKS        QString::fromAscii("PDF_HYPERLINKS")
-#define STR_SEARCHENGINE          QString::fromAscii("SEARCHENGINE")
-#define STR_HAVE_DOT              QString::fromAscii("HAVE_DOT")
-#define STR_CLASS_DIAGRAMS        QString::fromAscii("CLASS_DIAGRAMS")
-#define STR_CLASS_GRAPH           QString::fromAscii("CLASS_GRAPH")
-#define STR_COLLABORATION_GRAPH   QString::fromAscii("COLLABORATION_GRAPH")
-#define STR_GRAPHICAL_HIERARCHY   QString::fromAscii("GRAPHICAL_HIERARCHY")
-#define STR_INCLUDE_GRAPH         QString::fromAscii("INCLUDE_GRAPH")
-#define STR_INCLUDED_BY_GRAPH     QString::fromAscii("INCLUDED_BY_GRAPH")
-#define STR_CALL_GRAPH            QString::fromAscii("CALL_GRAPH")
-#define STR_CALLER_GRAPH          QString::fromAscii("CALLER_GRAPH")
-#define STR_HTML_COLORSTYLE_HUE   QString::fromAscii("HTML_COLORSTYLE_HUE")
-#define STR_HTML_COLORSTYLE_SAT   QString::fromAscii("HTML_COLORSTYLE_SAT")
-#define STR_HTML_COLORSTYLE_GAMMA QString::fromAscii("HTML_COLORSTYLE_GAMMA")
+#define STR_PROJECT_NAME          QString::fromLatin1("PROJECT_NAME")
+#define STR_PROJECT_LOGO          QString::fromLatin1("PROJECT_LOGO")
+#define STR_PROJECT_BRIEF         QString::fromLatin1("PROJECT_BRIEF")
+#define STR_INPUT                 QString::fromLatin1("INPUT")
+#define STR_OUTPUT_DIRECTORY      QString::fromLatin1("OUTPUT_DIRECTORY")
+#define STR_PROJECT_NUMBER        QString::fromLatin1("PROJECT_NUMBER")
+#define STR_RECURSIVE             QString::fromLatin1("RECURSIVE")
+#define STR_OPTIMIZE_OUTPUT_FOR_C QString::fromLatin1("OPTIMIZE_OUTPUT_FOR_C")
+#define STR_OPTIMIZE_OUTPUT_JAVA  QString::fromLatin1("OPTIMIZE_OUTPUT_JAVA")
+#define STR_OPTIMIZE_FOR_FORTRAN  QString::fromLatin1("OPTIMIZE_FOR_FORTRAN")
+#define STR_OPTIMIZE_OUTPUT_VHDL  QString::fromLatin1("OPTIMIZE_OUTPUT_VHDL")
+#define STR_OPTIMIZE_OUTPUT_SLICE QString::fromLatin1("OPTIMIZE_OUTPUT_SLICE")
+#define STR_CPP_CLI_SUPPORT       QString::fromLatin1("CPP_CLI_SUPPORT")
+#define STR_HIDE_SCOPE_NAMES      QString::fromLatin1("HIDE_SCOPE_NAMES")
+#define STR_EXTRACT_ALL           QString::fromLatin1("EXTRACT_ALL")
+#define STR_SOURCE_BROWSER        QString::fromLatin1("SOURCE_BROWSER")
+#define STR_GENERATE_HTML         QString::fromLatin1("GENERATE_HTML")
+#define STR_GENERATE_LATEX        QString::fromLatin1("GENERATE_LATEX")
+#define STR_GENERATE_MAN          QString::fromLatin1("GENERATE_MAN")
+#define STR_GENERATE_RTF          QString::fromLatin1("GENERATE_RTF")
+#define STR_GENERATE_XML          QString::fromLatin1("GENERATE_XML")
+#define STR_GENERATE_DOCBOOK      QString::fromLatin1("GENERATE_DOCBOOK")
+#define STR_GENERATE_HTMLHELP     QString::fromLatin1("GENERATE_HTMLHELP")
+#define STR_GENERATE_TREEVIEW     QString::fromLatin1("GENERATE_TREEVIEW")
+#define STR_USE_PDFLATEX          QString::fromLatin1("USE_PDFLATEX")
+#define STR_PDF_HYPERLINKS        QString::fromLatin1("PDF_HYPERLINKS")
+#define STR_SEARCHENGINE          QString::fromLatin1("SEARCHENGINE")
+#define STR_HAVE_DOT              QString::fromLatin1("HAVE_DOT")
+#define STR_CLASS_GRAPH           QString::fromLatin1("CLASS_GRAPH")
+#define STR_COLLABORATION_GRAPH   QString::fromLatin1("COLLABORATION_GRAPH")
+#define STR_GRAPHICAL_HIERARCHY   QString::fromLatin1("GRAPHICAL_HIERARCHY")
+#define STR_INCLUDE_GRAPH         QString::fromLatin1("INCLUDE_GRAPH")
+#define STR_INCLUDED_BY_GRAPH     QString::fromLatin1("INCLUDED_BY_GRAPH")
+#define STR_CALL_GRAPH            QString::fromLatin1("CALL_GRAPH")
+#define STR_CALLER_GRAPH          QString::fromLatin1("CALLER_GRAPH")
+#define STR_HTML_COLORSTYLE_HUE   QString::fromLatin1("HTML_COLORSTYLE_HUE")
+#define STR_HTML_COLORSTYLE_SAT   QString::fromLatin1("HTML_COLORSTYLE_SAT")
+#define STR_HTML_COLORSTYLE_GAMMA QString::fromLatin1("HTML_COLORSTYLE_GAMMA")
 
-static bool g_optimizeMapping[6][6] = 
+static bool g_optimizeMapping[7][7] =
 {
   // A: OPTIMIZE_OUTPUT_FOR_C
   // B: OPTIMIZE_OUTPUT_JAVA
@@ -52,23 +84,26 @@ static bool g_optimizeMapping[6][6] =
   // D: OPTIMIZE_OUTPUT_VHDL
   // E: CPP_CLI_SUPPORT
   // F: HIDE_SCOPE_NAMES
-  // A     B     C     D     E      F
-  { false,false,false,false,false,false }, // 0: C++
-  { false,false,false,false,true, false }, // 1: C++/CLI
-  { false,true, false,false,false,false }, // 2: C#/Java
-  { true, false,false,false,false,true  }, // 3: C/PHP 
-  { false,false,true, false,false,false }, // 4: Fortran
-  { false,false,false,true, false,false }, // 5: VHDL
+  // G: OPTIMIZE_OUTPUT_SLICE
+  // A     B     C     D     E     F     G
+  { false,false,false,false,false,false,false }, // 0: C++
+  { false,false,false,false,true, false,false }, // 1: C++/CLI
+  { false,true, false,false,false,false,false }, // 2: C#/Java
+  { true, false,false,false,false,true, false }, // 3: C/PHP
+  { false,false,true, false,false,false,false }, // 4: Fortran
+  { false,false,false,true, false,false,false }, // 5: VHDL
+  { false,false,false,false,false,false,true  }, // 6: SLICE
 };
 
-static QString g_optimizeOptionNames[6] =
+static QString g_optimizeOptionNames[7] =
 {
   STR_OPTIMIZE_OUTPUT_FOR_C,
   STR_OPTIMIZE_OUTPUT_JAVA,
   STR_OPTIMIZE_FOR_FORTRAN,
   STR_OPTIMIZE_OUTPUT_VHDL,
   STR_CPP_CLI_SUPPORT,
-  STR_HIDE_SCOPE_NAMES
+  STR_HIDE_SCOPE_NAMES,
+  STR_OPTIMIZE_OUTPUT_SLICE
 };
 
 //==========================================================================
@@ -76,30 +111,30 @@ static QString g_optimizeOptionNames[6] =
 static bool stringVariantToBool(const QVariant &v)
 {
   QString s = v.toString().toLower();
-  return s==QString::fromAscii("yes") || s==QString::fromAscii("true") || s==QString::fromAscii("1");
-} 
+  return s==QString::fromLatin1("yes") || s==QString::fromLatin1("true") || s==QString::fromLatin1("1");
+}
 
 static bool getBoolOption(
     const QHash<QString,Input*>&model,const QString &name)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   return stringVariantToBool(option->value());
-} 
+}
 
 static int getIntOption(
     const QHash<QString,Input*>&model,const QString &name)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   return option->value().toInt();
-} 
+}
 
 static QString getStringOption(
     const QHash<QString,Input*>&model,const QString &name)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   return option->value().toString();
 }
 
@@ -107,11 +142,11 @@ static void updateBoolOption(
     const QHash<QString,Input*>&model,const QString &name,bool bNew)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   bool bOld = stringVariantToBool(option->value());
   if (bOld!=bNew)
   {
-    option->value()=QString::fromAscii(bNew ? "true" : "false");
+    option->value()=QString::fromLatin1(bNew ? "true" : "false");
     option->update();
   }
 }
@@ -120,11 +155,11 @@ static void updateIntOption(
     const QHash<QString,Input*>&model,const QString &name,int iNew)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   int iOld = option->value().toInt();
   if (iOld!=iNew)
   {
-    option->value()=QString::fromAscii("%1").arg(iNew);
+    option->value()=QString::fromLatin1("%1").arg(iNew);
     option->update();
   }
 }
@@ -134,7 +169,7 @@ static void updateStringOption(
     const QHash<QString,Input*>&model,const QString &name,const QString &s)
 {
   Input *option = model[name];
-  Q_ASSERT(option!=0);
+  Q_ASSERT(option!=nullptr);
   if (option->value().toString()!=s)
   {
     option->value() = s;
@@ -148,11 +183,16 @@ TuneColorDialog::TuneColorDialog(int hue,int sat,int gamma,QWidget *parent) : QD
 {
    setWindowTitle(tr("Tune the color of the HTML output"));
    QGridLayout *layout = new QGridLayout(this);
-   m_image = new QImage(QString::fromAscii(":/images/tunecolor.png"));
-   m_imageLab = new QLabel;
+   m_imageLight        = QImage(QString::fromLatin1(":/images/tunecolor_light.png"));
+   m_imageDark         = QImage(QString::fromLatin1(":/images/tunecolor_dark.png"));
+   m_tab               = new QTabWidget;
+   m_imageLabLight     = new QLabel;
+   m_imageLabDark      = new QLabel;
+   m_tab->addTab(m_imageLabLight,QString::fromLatin1("light"));
+   m_tab->addTab(m_imageLabDark,QString::fromLatin1("dark"));
    updateImage(hue,sat,gamma);
    layout->addWidget(new QLabel(tr("Example output: use the sliders on the right to adjust the color")),0,0);
-   layout->addWidget(m_imageLab,1,0);
+   layout->addWidget(m_tab,1,0);
    QHBoxLayout *buttonsLayout = new QHBoxLayout;
 
    QPushButton *okButton = new QPushButton(tr("Ok"));
@@ -183,6 +223,7 @@ TuneColorDialog::TuneColorDialog(int hue,int sat,int gamma,QWidget *parent) : QD
    connect(huePicker,SIGNAL(newHsv(int,int,int)),this,SLOT(updateImage(int,int,int)));
    connect(satPicker,SIGNAL(newHsv(int,int,int)),this,SLOT(updateImage(int,int,int)));
    connect(gamPicker,SIGNAL(newHsv(int,int,int)),this,SLOT(updateImage(int,int,int)));
+   connect(m_tab,    SIGNAL(currentChanged(int)),this,SLOT(tabChanged(int)));
 
    buttonsLayout->addStretch();
    buttonsLayout->addWidget(okButton);
@@ -190,31 +231,24 @@ TuneColorDialog::TuneColorDialog(int hue,int sat,int gamma,QWidget *parent) : QD
    layout->addLayout(buttonsLayout,5,0,1,4);
 }
 
-void hsl2rgb(double h,double s,double l,
-             double *pRed,double *pGreen,double *pBlue)
+// convert color in HSL color space to RGB
+static constexpr void hsl2rgb(double h,double s,double l,
+                              double *pRed,double *pGreen,double *pBlue)
 {
-  double v;
-  double r,g,b;
-
-  r = l;   // default to gray
-  g = l;
-  b = l;
-  v = (l <= 0.5) ? (l * (1.0 + s)) : (l + s - l * s);
+  double r = l;   // default to gray
+  double g = l;
+  double b = l;
+  double v = (l <= 0.5) ? (l * (1.0 + s)) : (l + s - l * s);
   if (v > 0)
   {
-    double m;
-    double sv;
-    int sextant;
-    double fract, vsf, mid1, mid2;
-
-    m       = l + l - v;
-    sv      = (v - m ) / v;
-    h      *= 6.0;
-    sextant = (int)h;
-    fract   = h - sextant;
-    vsf     = v * sv * fract;
-    mid1    = m + vsf;
-    mid2    = v - vsf;
+    double m       = l + l - v;
+    double sv      = (v - m ) / v;
+    h             *= 6.0;
+    int sextant    = (int)h;
+    double fract   = h - sextant;
+    double vsf     = v * sv * fract;
+    double mid1    = m + vsf;
+    double mid2    = v - vsf;
     switch (sextant)
     {
       case 0:
@@ -255,21 +289,31 @@ void hsl2rgb(double h,double s,double l,
 }
 
 
+void TuneColorDialog::tabChanged(int current)
+{
+  // refresh image with the current settings when switching tabs
+  updateImage(m_hue,m_sat,m_gam);
+}
 
 void TuneColorDialog::updateImage(int hue,int sat,int gam)
 {
-  QImage coloredImg(m_image->width(),m_image->height(),QImage::Format_RGB32);
-  uint *srcPixel = (uint *)m_image->scanLine(0);
+  QImage *image = m_tab->currentIndex()==0 ? &m_imageLight : &m_imageDark;
+  QLabel *label = m_tab->currentIndex()==0 ? m_imageLabLight : m_imageLabDark;
+  QImage coloredImg(image->width(),image->height(),QImage::Format_RGB32);
+  uint *srcPixel = (uint *)image->scanLine(0);
   uint *dstPixel = (uint *)coloredImg.scanLine(0);
   uint nrPixels = coloredImg.width()*coloredImg.height();
+  double r,g,b;
   for (uint i=0;i<nrPixels;i++,srcPixel++,dstPixel++)
   {
     QColor c = QColor::fromRgb(*srcPixel);
-    double r,g,b;
     hsl2rgb(hue/359.0, sat/255.0, pow(c.green()/255.0,gam/100.0),&r,&g,&b);
     *dstPixel = qRgb((int)(r*255.0),(int)(g*255.0),(int)(b*255.0));
   }
-  m_imageLab->setPixmap(QPixmap::fromImage(coloredImg));
+
+  QPixmap pm = QPixmap::fromImage(coloredImg);
+  pm.setDevicePixelRatio(2.0);
+  label->setPixmap(pm);
   m_hue = hue;
   m_sat = sat;
   m_gam = gam;
@@ -298,7 +342,7 @@ ColorPicker::ColorPicker(Mode m)
   m_gam = 100;
   m_sat = 100;
   m_mode = m;
-  m_pix = 0;
+  m_pix = nullptr;
 }
 
 ColorPicker::~ColorPicker()
@@ -313,19 +357,19 @@ void ColorPicker::paintEvent(QPaintEvent*)
   QRect r(0, foff, w, height() - 2*foff);
   int wi = r.width() - 2;
   int hi = r.height() - 2;
-  if (!m_pix || m_pix->height() != hi || m_pix->width() != wi) 
+  if (!m_pix || m_pix->height() != hi || m_pix->width() != wi)
   {
     delete m_pix;
     QImage img(wi, hi, QImage::Format_RGB32);
     int y;
     uint *pixel = (uint *) img.scanLine(0);
-    for (y = 0; y < hi; y++) 
+    for (y = 0; y < hi; y++)
     {
       const uint *end = pixel + wi;
       int yh = y2hue(y+coff);
       int ys = y2sat(y+coff);
       int yg = y2gam(y+coff);
-      while (pixel < end) 
+      while (pixel < end)
       {
         QColor c;
         c.setHsv(yh, ys, (int)(255*pow(0.7,yg/100.0)));
@@ -339,10 +383,10 @@ void ColorPicker::paintEvent(QPaintEvent*)
   p.drawPixmap(1, coff, *m_pix);
   const QPalette &g = palette();
   qDrawShadePanel(&p, r, g, true);
-  p.setPen(g.foreground().color());
-  p.setBrush(g.foreground());
+  p.setPen(g.windowText().color());
+  p.setBrush(g.windowText());
   QPolygon a;
-  int y = m_mode==Hue ?        hue2y(m_hue) : 
+  int y = m_mode==Hue ?        hue2y(m_hue) :
           m_mode==Saturation ? sat2y(m_sat) :
                                gam2y(m_gam);
   a.setPoints(3, w, y, w+5, y+5, w+5, y-5);
@@ -352,23 +396,23 @@ void ColorPicker::paintEvent(QPaintEvent*)
 
 void ColorPicker::mouseMoveEvent(QMouseEvent *m)
 {
-  if      (m_mode==Hue)        setHue(y2hue(m->y())); 
-  else if (m_mode==Saturation) setSat(y2sat(m->y()));
-  else                         setGam(y2gam(m->y()));
+  if      (m_mode==Hue)        setHue(y2hue(getMouseYPositionFromEvent(m)));
+  else if (m_mode==Saturation) setSat(y2sat(getMouseYPositionFromEvent(m)));
+  else                         setGam(y2gam(getMouseYPositionFromEvent(m)));
 }
 
 void ColorPicker::mousePressEvent(QMouseEvent *m)
 {
-  if      (m_mode==Hue)        setHue(y2hue(m->y())); 
-  else if (m_mode==Saturation) setSat(y2sat(m->y()));
-  else                         setGam(y2gam(m->y()));
+  if      (m_mode==Hue)        setHue(y2hue(getMouseYPositionFromEvent(m)));
+  else if (m_mode==Saturation) setSat(y2sat(getMouseYPositionFromEvent(m)));
+  else                         setGam(y2gam(getMouseYPositionFromEvent(m)));
 }
 
 void ColorPicker::setHue(int h)
 {
   if (h==m_hue) return;
   m_hue = qMax(0,qMin(h,359));
-  delete m_pix; m_pix=0;
+  delete m_pix; m_pix=nullptr;
   repaint();
   emit newHsv(m_hue,m_sat,m_gam);
 }
@@ -377,7 +421,7 @@ void ColorPicker::setSat(int s)
 {
   if (s==m_sat) return;
   m_sat = qMax(0,qMin(s,255));
-  delete m_pix; m_pix=0;
+  delete m_pix; m_pix=nullptr;
   repaint();
   emit newHsv(m_hue,m_sat,m_gam);
 }
@@ -386,7 +430,7 @@ void ColorPicker::setGam(int g)
 {
   if (g==m_gam) return;
   m_gam = qMax(40,qMin(g,240));
-  delete m_pix; m_pix=0;
+  delete m_pix; m_pix=nullptr;
   repaint();
   emit newHsv(m_hue,m_sat,m_gam);
 }
@@ -398,7 +442,7 @@ void ColorPicker::setCol(int h, int s, int g)
     m_hue = h;
     m_sat = s;
     m_gam = g;
-    delete m_pix; m_pix=0;
+    delete m_pix; m_pix=nullptr;
     repaint();
   }
 }
@@ -444,7 +488,7 @@ int ColorPicker::gam2y(int g)
 Step1::Step1(Wizard *wizard,const QHash<QString,Input*> &modelData) : m_wizard(wizard), m_modelData(modelData)
 {
   QVBoxLayout *layout = new QVBoxLayout(this);
-  layout->setMargin(4);
+  layout->setContentsMargins(4,4,4,4);
   layout->setSpacing(8);
   QLabel *l = new QLabel(this);
   l->setText(tr("Provide some information "
@@ -498,9 +542,10 @@ Step1::Step1(Wizard *wizard,const QHash<QString,Input*> &modelData) : m_wizard(w
 
   //---------------------------------------------------
   QFrame *f = new QFrame( this );
-  f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
+  f->setFrameShape(QFrame::HLine);
+  f->setFrameShadow(QFrame::Sunken);
   layout->addWidget(f);
-  
+
   l = new QLabel(this);
   l->setText(tr("Specify the directory to scan for source code"));
   layout->addWidget(l);
@@ -519,12 +564,13 @@ Step1::Step1(Wizard *wizard,const QHash<QString,Input*> &modelData) : m_wizard(w
 
   m_recursive = new QCheckBox(this);
   m_recursive->setText(tr("Scan recursively"));
-  m_recursive->setChecked(TRUE);
+  m_recursive->setChecked(true);
   layout->addWidget(m_recursive);
 
   //---------------------------------------------------
   f = new QFrame( this );
-  f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
+  f->setFrameShape(QFrame::HLine);
+  f->setFrameShadow(QFrame::Sunken);
   layout->addWidget(f);
 
   l = new QLabel(this);
@@ -572,9 +618,9 @@ void Step1::selectProjectIcon()
   else
   {
     QFile Fout(iconName);
-    if(!Fout.exists()) 
+    if(!Fout.exists())
     {
-      m_projIconLab->setText(tr("Sorry, cannot find file(")+iconName+QString::fromAscii(");"));
+      m_projIconLab->setText(tr("Sorry, cannot find file(")+iconName+QString::fromLatin1(");"));
     }
     else
     {
@@ -585,7 +631,7 @@ void Step1::selectProjectIcon()
       }
       else
       {
-        m_projIconLab->setText(tr("Sorry, no preview available (")+iconName+QString::fromAscii(");"));
+        m_projIconLab->setText(tr("Sorry, no preview available (")+iconName+QString::fromLatin1(");"));
       }
     }
   }
@@ -604,7 +650,7 @@ void Step1::selectSourceDir()
   }
   if (dirName.isEmpty())
   {
-    dirName=QString::fromAscii(".");
+    dirName=QString::fromLatin1(".");
   }
   m_sourceDir->setText(dirName);
 }
@@ -621,7 +667,7 @@ void Step1::selectDestinationDir()
   }
   if (dirName.isEmpty())
   {
-    dirName=QString::fromAscii(".");
+    dirName=QString::fromLatin1(".");
   }
   m_destDir->setText(dirName);
 }
@@ -681,9 +727,9 @@ void Step1::init()
   if (!iconName.isEmpty())
   {
     QFile Fout(iconName);
-    if(!Fout.exists()) 
+    if(!Fout.exists())
     {
-      m_projIconLab->setText(tr("Sorry, cannot find file(")+iconName+QString::fromAscii(");"));
+      m_projIconLab->setText(tr("Sorry, cannot find file(")+iconName+QString::fromLatin1(");"));
     }
     else
     {
@@ -694,7 +740,7 @@ void Step1::init()
       }
       else
       {
-        m_projIconLab->setText(tr("Sorry, no preview available (")+iconName+QString::fromAscii(");"));
+        m_projIconLab->setText(tr("Sorry, no preview available (")+iconName+QString::fromLatin1(");"));
       }
     }
   }
@@ -715,7 +761,7 @@ void Step1::init()
 
 //==========================================================================
 
-Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData) 
+Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   : m_wizard(wizard), m_modelData(modelData)
 {
   QRadioButton *r;
@@ -740,17 +786,18 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   // m_crossRef -> SOURCE_BROWSER = YES/NO
   gbox->addWidget(m_crossRef,3,0);
   layout->addWidget(m_extractMode);
-  
+
   //---------------------------------------------------
   QFrame *f = new QFrame( this );
-  f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
+  f->setFrameShape(QFrame::HLine);
+  f->setFrameShadow(QFrame::Sunken);
   layout->addWidget(f);
 
   m_optimizeLangGroup = new QButtonGroup(this);
   m_optimizeLang = new QGroupBox(this);
   m_optimizeLang->setTitle(tr("Select programming language to optimize the results for"));
-  gbox = new QGridLayout( m_optimizeLang ); 
-  
+  gbox = new QGridLayout( m_optimizeLang );
+
   r = new QRadioButton(m_optimizeLang);
   r->setText(tr("Optimize for C++ output"));
   r->setChecked(true);
@@ -761,6 +808,7 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   //      OPTIMIZE_OUTPUT_VHDL  = NO
   //      CPP_CLI_SUPPORT       = NO
   //      HIDE_SCOPE_NAMES      = NO
+  //      OPTIMIZE_OUTPUT_SLICE = NO
   gbox->addWidget(r,0,0);
   r = new QRadioButton(tr("Optimize for C++/CLI output"));
   gbox->addWidget(r,1,0);
@@ -771,6 +819,7 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   //      OPTIMIZE_OUTPUT_VHDL  = NO
   //      CPP_CLI_SUPPORT       = YES
   //      HIDE_SCOPE_NAMES      = NO
+  //      OPTIMIZE_OUTPUT_SLICE = NO
   r = new QRadioButton(tr("Optimize for Java or C# output"));
   m_optimizeLangGroup->addButton(r, 2);
   // 2 -> OPTIMIZE_OUTPUT_FOR_C = NO
@@ -779,6 +828,7 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   //      OPTIMIZE_OUTPUT_VHDL  = NO
   //      CPP_CLI_SUPPORT       = NO
   //      HIDE_SCOPE_NAMES      = NO
+  //      OPTIMIZE_OUTPUT_SLICE = NO
   gbox->addWidget(r,2,0);
   r = new QRadioButton(tr("Optimize for C or PHP output"));
   m_optimizeLangGroup->addButton(r, 3);
@@ -788,6 +838,7 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   //      OPTIMIZE_OUTPUT_VHDL  = NO
   //      CPP_CLI_SUPPORT       = NO
   //      HIDE_SCOPE_NAMES      = YES
+  //      OPTIMIZE_OUTPUT_SLICE = NO
   gbox->addWidget(r,3,0);
   r = new QRadioButton(tr("Optimize for Fortran output"));
   m_optimizeLangGroup->addButton(r, 4);
@@ -797,6 +848,7 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   //      OPTIMIZE_OUTPUT_VHDL  = NO
   //      CPP_CLI_SUPPORT       = NO
   //      HIDE_SCOPE_NAMES      = NO
+  //      OPTIMIZE_OUTPUT_SLICE = NO
   gbox->addWidget(r,4,0);
   r = new QRadioButton(tr("Optimize for VHDL output"));
   m_optimizeLangGroup->addButton(r, 5);
@@ -806,23 +858,41 @@ Step2::Step2(Wizard *wizard,const QHash<QString,Input*> &modelData)
   //      OPTIMIZE_OUTPUT_VHDL  = YES
   //      CPP_CLI_SUPPORT       = NO
   //      HIDE_SCOPE_NAMES      = NO
+  //      OPTIMIZE_OUTPUT_SLICE = NO
   gbox->addWidget(r,5,0);
+  r = new QRadioButton(tr("Optimize for SLICE output"));
+  m_optimizeLangGroup->addButton(r, 6);
+  // 5 -> OPTIMIZE_OUTPUT_FOR_C = NO
+  //      OPTIMIZE_OUTPUT_JAVA  = NO
+  //      OPTIMIZE_FOR_FORTRAN  = NO
+  //      OPTIMIZE_OUTPUT_VHDL  = NO
+  //      CPP_CLI_SUPPORT       = NO
+  //      HIDE_SCOPE_NAMES      = NO
+  //      OPTIMIZE_OUTPUT_SLICE = YES
+  gbox->addWidget(r,6,0);
 
   layout->addWidget(m_optimizeLang);
   layout->addStretch(1);
 
   connect(m_crossRef,SIGNAL(stateChanged(int)),
           SLOT(changeCrossRefState(int)));
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
   connect(m_optimizeLangGroup,SIGNAL(buttonClicked(int)),
           SLOT(optimizeFor(int)));
   connect(m_extractModeGroup,SIGNAL(buttonClicked(int)),
           SLOT(extractMode(int)));
+#else
+  connect(m_optimizeLangGroup,SIGNAL(idClicked(int)),
+          SLOT(optimizeFor(int)));
+  connect(m_extractModeGroup,SIGNAL(idClicked(int)),
+          SLOT(extractMode(int)));
+#endif
 }
 
 
 void Step2::optimizeFor(int choice)
 {
-  for (int i=0;i<6;i++)
+  for (int i=0;i<7;i++)
   {
     updateBoolOption(m_modelData,
                      g_optimizeOptionNames[i],
@@ -852,16 +922,17 @@ void Step2::init()
   else if (getBoolOption(m_modelData,STR_OPTIMIZE_OUTPUT_FOR_C)) x=3;
   else if (getBoolOption(m_modelData,STR_OPTIMIZE_FOR_FORTRAN))  x=4;
   else if (getBoolOption(m_modelData,STR_OPTIMIZE_OUTPUT_VHDL))  x=5;
+  else if (getBoolOption(m_modelData,STR_OPTIMIZE_OUTPUT_SLICE)) x=6;
   m_optimizeLangGroup->button(x)->setChecked(true);
 }
 
 //==========================================================================
 
-Step3::Step3(Wizard *wizard,const QHash<QString,Input*> &modelData) 
+Step3::Step3(Wizard *wizard,const QHash<QString,Input*> &modelData)
   : m_wizard(wizard), m_modelData(modelData)
 {
-  QVBoxLayout *vbox = 0;
-  QRadioButton *r   = 0;
+  QVBoxLayout *vbox = nullptr;
+  QRadioButton *r   = nullptr;
 
   QGridLayout *gbox = new QGridLayout( this );
   gbox->addWidget(new QLabel(tr("Select the output format(s) to generate")),0,0);
@@ -927,21 +998,28 @@ Step3::Step3(Wizard *wizard,const QHash<QString,Input*> &modelData)
   // GENERATE_RTF
   m_xmlEnabled=new QCheckBox(tr("XML"));
   // GENERATE_XML
+  m_docbookEnabled=new QCheckBox(tr("Docbook"));
+  // GENERATE_DOCBOOK
   gbox->addWidget(m_manEnabled,3,0);
   gbox->addWidget(m_rtfEnabled,4,0);
   gbox->addWidget(m_xmlEnabled,5,0);
+  gbox->addWidget(m_docbookEnabled,6,0);
 
-  gbox->setRowStretch(6,1);
+  gbox->setRowStretch(7,1);
   connect(m_htmlOptions,SIGNAL(toggled(bool)),SLOT(setHtmlEnabled(bool)));
   connect(m_texOptions,SIGNAL(toggled(bool)),SLOT(setLatexEnabled(bool)));
   connect(m_manEnabled,SIGNAL(stateChanged(int)),SLOT(setManEnabled(int)));
   connect(m_rtfEnabled,SIGNAL(stateChanged(int)),SLOT(setRtfEnabled(int)));
   connect(m_xmlEnabled,SIGNAL(stateChanged(int)),SLOT(setXmlEnabled(int)));
+  connect(m_docbookEnabled,SIGNAL(stateChanged(int)),SLOT(setDocbookEnabled(int)));
   connect(m_searchEnabled,SIGNAL(stateChanged(int)),SLOT(setSearchEnabled(int)));
-  connect(m_htmlOptionsGroup,SIGNAL(buttonClicked(int)),
-          SLOT(setHtmlOptions(int)));
-  connect(m_texOptionsGroup,SIGNAL(buttonClicked(int)),
-          SLOT(setLatexOptions(int)));
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+  connect(m_htmlOptionsGroup,SIGNAL(buttonClicked(int)),SLOT(setHtmlOptions(int)));
+  connect(m_texOptionsGroup,SIGNAL(buttonClicked(int)),SLOT(setLatexOptions(int)));
+#else
+  connect(m_htmlOptionsGroup,SIGNAL(idClicked(int)),SLOT(setHtmlOptions(int)));
+  connect(m_texOptionsGroup,SIGNAL(idClicked(int)),SLOT(setLatexOptions(int)));
+#endif
   connect(m_tuneColor,SIGNAL(clicked()),SLOT(tuneColorDialog()));
 }
 
@@ -984,6 +1062,11 @@ void Step3::setXmlEnabled(int state)
   updateBoolOption(m_modelData,STR_GENERATE_XML,state==Qt::Checked);
 }
 
+void Step3::setDocbookEnabled(int state)
+{
+  updateBoolOption(m_modelData,STR_GENERATE_DOCBOOK,state==Qt::Checked);
+}
+
 void Step3::setSearchEnabled(int state)
 {
   updateBoolOption(m_modelData,STR_SEARCHENGINE,state==Qt::Checked);
@@ -1034,6 +1117,7 @@ void Step3::init()
   m_manEnabled->setChecked(getBoolOption(m_modelData,STR_GENERATE_MAN));
   m_rtfEnabled->setChecked(getBoolOption(m_modelData,STR_GENERATE_RTF));
   m_xmlEnabled->setChecked(getBoolOption(m_modelData,STR_GENERATE_XML));
+  m_docbookEnabled->setChecked(getBoolOption(m_modelData,STR_GENERATE_DOCBOOK));
   m_searchEnabled->setChecked(getBoolOption(m_modelData,STR_SEARCHENGINE));
   if (getBoolOption(m_modelData,STR_GENERATE_HTMLHELP))
   {
@@ -1063,30 +1147,37 @@ void Step3::init()
 
 //==========================================================================
 
-Step4::Step4(Wizard *wizard,const QHash<QString,Input*> &modelData) 
+Step4::Step4(Wizard *wizard,const QHash<QString,Input*> &modelData)
   : m_wizard(wizard), m_modelData(modelData)
 {
   m_diagramModeGroup = new QButtonGroup(this);
   QGridLayout *gbox = new QGridLayout( this );
   gbox->addWidget(new QLabel(tr("Diagrams to generate")),0,0);
 
+  // CLASS_GRAPH = NO, HAVE_DOT = NO
   QRadioButton *rb = new QRadioButton(tr("No diagrams"));
   m_diagramModeGroup->addButton(rb, 0);
   gbox->addWidget(rb,1,0);
-  // CLASS_DIAGRAMS = NO, HAVE_DOT = NO
   rb->setChecked(true);
-  rb = new QRadioButton(tr("Use built-in class diagram generator"));
+
+  // CLASS_GRAPH = TEXT, HAVE_DOT = NO
+  rb = new QRadioButton(tr("Text only"));
   m_diagramModeGroup->addButton(rb, 1);
-  // CLASS_DIAGRAMS = YES, HAVE_DOT = NO
   gbox->addWidget(rb,2,0);
-  rb = new QRadioButton(tr("Use dot tool from the GraphViz package"));
+
+  // CLASS_GRAPH = YES/GRAPH, HAVE_DOT = NO
+  rb = new QRadioButton(tr("Use built-in class diagram generator"));
   m_diagramModeGroup->addButton(rb, 2);
   gbox->addWidget(rb,3,0);
-  // CLASS_DIAGRAMS = NO, HAVE_DOT = YES
+
+  // CLASS_GRAPH = YES/GRAPH, HAVE_DOT = YES
+  rb = new QRadioButton(tr("Use dot tool from the GraphViz package"));
+  m_diagramModeGroup->addButton(rb, 3);
+  gbox->addWidget(rb,4,0);
 
   m_dotGroup = new QGroupBox(tr("Dot graphs to generate"));
     QVBoxLayout *vbox = new QVBoxLayout;
-    m_dotClass=new QCheckBox(tr("Class diagrams"));
+    m_dotClass=new QCheckBox(tr("Class graphs"));
     // CLASS_GRAPH
     m_dotCollaboration=new QCheckBox(tr("Collaboration diagrams"));
     // COLLABORATION_GRAPH
@@ -1111,14 +1202,19 @@ Step4::Step4(Wizard *wizard,const QHash<QString,Input*> &modelData)
     m_dotGroup->setLayout(vbox);
     m_dotClass->setChecked(true);
     m_dotGroup->setEnabled(false);
-  gbox->addWidget(m_dotGroup,4,0);
+  gbox->addWidget(m_dotGroup,5,0);
 
   m_dotInclude->setChecked(true);
   m_dotCollaboration->setChecked(true);
-  gbox->setRowStretch(5,1);
+  gbox->setRowStretch(6,1);
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
   connect(m_diagramModeGroup,SIGNAL(buttonClicked(int)),
           this,SLOT(diagramModeChanged(int)));
+#else
+  connect(m_diagramModeGroup,SIGNAL(idClicked(int)),
+          this,SLOT(diagramModeChanged(int)));
+#endif
   connect(m_dotClass,SIGNAL(stateChanged(int)),
           this,SLOT(setClassGraphEnabled(int)));
   connect(m_dotCollaboration,SIGNAL(stateChanged(int)),
@@ -1140,24 +1236,37 @@ void Step4::diagramModeChanged(int id)
   if (id==0) // no diagrams
   {
     updateBoolOption(m_modelData,STR_HAVE_DOT,false);
-    updateBoolOption(m_modelData,STR_CLASS_DIAGRAMS,false);
+    updateStringOption(m_modelData,STR_CLASS_GRAPH, QString::fromLatin1("NO"));
   }
-  else if (id==1) // builtin diagrams
+  else if (id==1) // text only
   {
     updateBoolOption(m_modelData,STR_HAVE_DOT,false);
-    updateBoolOption(m_modelData,STR_CLASS_DIAGRAMS,true);
+    updateStringOption(m_modelData,STR_CLASS_GRAPH, QString::fromLatin1("TEXT"));
   }
-  else if (id==2) // dot diagrams
+  else if (id==2) // builtin diagrams
+  {
+    updateBoolOption(m_modelData,STR_HAVE_DOT,false);
+    updateStringOption(m_modelData,STR_CLASS_GRAPH, QString::fromLatin1("YES"));
+  }
+  else if (id==3) // dot diagrams
   {
     updateBoolOption(m_modelData,STR_HAVE_DOT,true);
-    updateBoolOption(m_modelData,STR_CLASS_DIAGRAMS,false);
+    updateStringOption(m_modelData,STR_CLASS_GRAPH, QString::fromLatin1("YES"));
   }
-  m_dotGroup->setEnabled(id==2);
+  m_dotGroup->setEnabled(id==3);
 }
 
 void Step4::setClassGraphEnabled(int state)
 {
-  updateBoolOption(m_modelData,STR_CLASS_GRAPH,state==Qt::Checked);
+  QString classGraph = getStringOption(m_modelData,STR_CLASS_GRAPH);
+  if (state==Qt::Checked)
+  {
+    updateStringOption(m_modelData,STR_CLASS_GRAPH,QString::fromLatin1("YES"));
+  }
+  else if (classGraph==QString::fromLatin1("YES") || classGraph==QString::fromLatin1("GRAPH"))
+  {
+    updateStringOption(m_modelData,STR_CLASS_GRAPH,QString::fromLatin1("NO"));
+  }
 }
 
 void Step4::setCollaborationGraphEnabled(int state)
@@ -1192,18 +1301,22 @@ void Step4::setCallerGraphEnabled(int state)
 
 void Step4::init()
 {
-  if (getBoolOption(m_modelData,STR_HAVE_DOT))
+  int id = 0;
+  QString classGraph = getStringOption(m_modelData,STR_CLASS_GRAPH).toLower();
+  if (classGraph==QString::fromLatin1("yes") || classGraph==QString::fromLatin1("graph"))
   {
-    m_diagramModeGroup->button(2)->setChecked(true); // Dot
+    id = getBoolOption(m_modelData,STR_HAVE_DOT) ? 3 : 2;
   }
-  else if (getBoolOption(m_modelData,STR_CLASS_DIAGRAMS))
+  else if (classGraph==QString::fromLatin1("text"))
   {
-    m_diagramModeGroup->button(1)->setChecked(true); // Builtin diagrams
+    id = 1;
   }
   else
   {
-    m_diagramModeGroup->button(0)->setChecked(true); // no diagrams
+    id = 0;
   }
+  m_diagramModeGroup->button(id)->setChecked(true); // no diagrams
+  m_dotGroup->setEnabled(id==3);
   m_dotClass->setChecked(getBoolOption(m_modelData,STR_CLASS_GRAPH));
   m_dotCollaboration->setChecked(getBoolOption(m_modelData,STR_COLLABORATION_GRAPH));
   m_dotInheritance->setChecked(getBoolOption(m_modelData,STR_GRAPHICAL_HIERARCHY));
@@ -1215,17 +1328,17 @@ void Step4::init()
 
 //==========================================================================
 
-Wizard::Wizard(const QHash<QString,Input*> &modelData, QWidget *parent) : 
+Wizard::Wizard(const QHash<QString,Input*> &modelData, QWidget *parent) :
   QSplitter(parent), m_modelData(modelData)
 {
   m_treeWidget = new QTreeWidget;
   m_treeWidget->setColumnCount(1);
-  m_treeWidget->setHeaderLabels(QStringList() << QString::fromAscii("Topics"));
+  m_treeWidget->setHeaderLabels(QStringList() << QString::fromLatin1("Topics"));
   QList<QTreeWidgetItem*> items;
-  items.append(new QTreeWidgetItem((QTreeWidget*)0,QStringList(tr("Project"))));
-  items.append(new QTreeWidgetItem((QTreeWidget*)0,QStringList(tr("Mode"))));
-  items.append(new QTreeWidgetItem((QTreeWidget*)0,QStringList(tr("Output"))));
-  items.append(new QTreeWidgetItem((QTreeWidget*)0,QStringList(tr("Diagrams"))));
+  items.append(new QTreeWidgetItem((QTreeWidget*)nullptr,QStringList(tr("Project"))));
+  items.append(new QTreeWidgetItem((QTreeWidget*)nullptr,QStringList(tr("Mode"))));
+  items.append(new QTreeWidgetItem((QTreeWidget*)nullptr,QStringList(tr("Output"))));
+  items.append(new QTreeWidgetItem((QTreeWidget*)nullptr,QStringList(tr("Diagrams"))));
   m_treeWidget->insertTopLevelItems(0,items);
 
   m_topicStack = new QStackedWidget;
@@ -1268,7 +1381,7 @@ void Wizard::activateTopic(QTreeWidgetItem *item,QTreeWidgetItem *)
 {
   if (item)
   {
-    
+
     QString label = item->text(0);
     if (label==tr("Project"))
     {
