@@ -55,7 +55,13 @@ class XMLHandlers
 class XMLLocator
 {
   public:
-    virtual ~XMLLocator() {}
+    XMLLocator() = default;
+    XMLLocator(const XMLLocator &) = delete;
+    XMLLocator &operator=(const XMLLocator &) = delete;
+    XMLLocator(XMLLocator &&) = delete;
+    XMLLocator &operator=(XMLLocator &&) = delete;
+    virtual ~XMLLocator() = default;
+
     virtual int lineNr() const = 0;
     virtual std::string fileName() const = 0;
 };
@@ -71,7 +77,11 @@ class XMLParser : public XMLLocator
      */
     XMLParser(const XMLHandlers &handlers);
     /*! Destructor */
-   ~XMLParser();
+   ~XMLParser() override;
+    XMLParser(const XMLParser &) = delete;
+    XMLParser &operator=(const XMLParser &) = delete;
+    XMLParser(XMLParser &&) = delete;
+    XMLParser &operator=(XMLParser &&) = delete;
 
    using Transcode = bool(std::string &,const char *);
 
@@ -93,8 +103,8 @@ class XMLParser : public XMLLocator
               );
 
   private:
-   virtual int lineNr() const override;
-   virtual std::string fileName() const override;
+   int lineNr() const override;
+   std::string fileName() const override;
    struct Private;
    std::unique_ptr<Private> p;
 };
