@@ -22,9 +22,14 @@
 //! @brief Private interface between Parser (constexp.y) and Lexer (constexp.l)
 
 #include "cppvalue.h"
+#ifdef YYSTYPE
+#undef YYSTYPE
+#endif
 #define YYSTYPE CPPValue
+#define YY_TYPEDEF_YY_SCANNER_T
 
-typedef void* yyscan_t;
+struct yyguts_t;
+typedef yyguts_t* yyscan_t;
 struct constexpYY_state
 {
   std::string  strToken;
@@ -32,13 +37,14 @@ struct constexpYY_state
   int          constExpLineNr;
   std::string  constExpFileName;
 
+  std::string orgString;
   std::string inputString;
   int         inputPosition;
 };
 constexpYY_state* constexpYYget_extra(yyscan_t yyscanner );
 
-extern int constexpYYlex(YYSTYPE *lvalp, yyscan_t);
-extern int constexpYYparse(yyscan_t);
+int constexpYYlex(YYSTYPE *lvalp, yyscan_t);
+int constexpYYparse(yyscan_t);
 
 
 #endif
