@@ -80,44 +80,57 @@ inline void writeDocbookString(TextStream &t,const QCString &s)
   t << convertToDocBook(s);
 }
 
-inline void writeDocbookCodeString(TextStream &t,const QCString &str, size_t &col)
+inline void writeDocbookCodeString(bool hide,TextStream &t,const QCString &str, size_t &col, size_t stripIndentAmount)
 {
   if (str.isEmpty()) return;
+  const int tabSize = Config_getInt(TAB_SIZE);
   const char *s = str.data();
   char c=0;
-  while ((c=*s++))
+  if (hide)
   {
-    switch(c)
+    col = updateColumnCount(s,col);
+  }
+  else
+  {
+    while ((c=*s++))
     {
-      case '\t':
-        {
-          int tabSize = Config_getInt(TAB_SIZE);
-          int spacesToNextTabStop = tabSize - (col%tabSize);
-          col+=spacesToNextTabStop;
-          while (spacesToNextTabStop--) t << "&#32;";
-          break;
-        }
-      case ' ':  t << "&#32;"; col++;  break;
-      case '<':  t << "&lt;"; col++;   break;
-      case '>':  t << "&gt;"; col++;   break;
-      case '&':  t << "&amp;"; col++;  break;
-      case '\'': t << "&apos;"; col++; break;
-      case '"':  t << "&quot;"; col++; break;
-      default:
-        {
-          uint8_t uc = static_cast<uint8_t>(c);
-          static const char *hex="0123456789ABCDEF";
-          if (uc<32)
+      switch(c)
+      {
+        case '\t':
           {
-            t << "&#x24" << hex[uc>>4] << hex[uc&0xF] << ";";
-          }
-          else
-          {
-            t << c;
+            int spacesToNextTabStop = tabSize - (col%tabSize);
+            while (spacesToNextTabStop--)
+            {
+              if (col>=stripIndentAmount) t << "&#32;";
+              col++;
+            }
+            break;
           }
+        case ' ':
+          if (col>=stripIndentAmount) t << "&#32;";
           col++;
-        }
-        break;
+          break;
+        case '<':  t << "&lt;"; col++;   break;
+        case '>':  t << "&gt;"; col++;   break;
+        case '&':  t << "&amp;"; col++;  break;
+        case '\'': t << "&apos;"; col++; break;
+        case '"':  t << "&quot;"; col++; break;
+        default:
+                   {
+                     uint8_t uc = static_cast<uint8_t>(c);
+                     static const char *hex="0123456789ABCDEF";
+                     if (uc<32)
+                     {
+                       t << "&#x24" << hex[uc>>4] << hex[uc&0xF] << ";";
+                     }
+                     else
+                     {
+                       t << c;
+                     }
+                     col++;
+                   }
+                   break;
+      }
     }
   }
 }
@@ -153,7 +166,27 @@ DocbookCodeGenerator::DocbookCodeGenerator(TextStream *t) : m_t(t)
 void DocbookCodeGenerator::codify(const QCString &text)
 {
   Docbook_DB(("(codify \"%s\")\n",text));
-  writeDocbookCodeString(*m_t,text,m_col);
+  writeDocbookCodeString(m_hide,*m_t,text,m_col,static_cast<size_t>(m_stripIndentAmount));
+}
+
+void DocbookCodeGenerator::stripCodeComments(bool b)
+{
+  m_stripCodeComments = b;
+}
+
+void DocbookCodeGenerator::startSpecialComment()
+{
+  m_hide = m_stripCodeComments;
+}
+
+void DocbookCodeGenerator::endSpecialComment()
+{
+  m_hide = false;
+}
+
+void DocbookCodeGenerator::setStripIndentAmount(size_t amount)
+{
+  m_stripIndentAmount = amount;
 }
 
 void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
@@ -161,6 +194,7 @@ void DocbookCodeGenerator::writeCodeLink(CodeSymbolType,
     const QCString &anchor,const QCString &name,
     const QCString &tooltip)
 {
+  if (m_hide) return;
   Docbook_DB(("(writeCodeLink)\n"));
   writeDocbookLink(*m_t,ref,file,anchor,name,tooltip);
   m_col+=name.length();
@@ -171,6 +205,7 @@ void DocbookCodeGenerator::writeCodeLinkLine(CodeSymbolType,
     const QCString &,const QCString &name,
     const QCString &,bool writeLineAnchor)
 {
+  if (m_hide) return;
   Docbook_DB(("(writeCodeLinkLine)\n"));
   if (!writeLineAnchor) return;
   *m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
@@ -189,6 +224,7 @@ void DocbookCodeGenerator::writeTooltip(const QCString &, const DocLinkInfo &, c
 
 void DocbookCodeGenerator::startCodeLine(int)
 {
+  if (m_hide) return;
   Docbook_DB(("(startCodeLine)\n"));
   m_insideCodeLine=TRUE;
   m_col=0;
@@ -196,6 +232,7 @@ void DocbookCodeGenerator::startCodeLine(int)
 
 void DocbookCodeGenerator::endCodeLine()
 {
+  if (m_hide) return;
   if (m_insideCodeLine) *m_t << "\n";
   Docbook_DB(("(endCodeLine)\n"));
   m_lineNumber = -1;
@@ -206,6 +243,7 @@ void DocbookCodeGenerator::endCodeLine()
 
 void DocbookCodeGenerator::startFontClass(const QCString &colorClass)
 {
+  if (m_hide) return;
   Docbook_DB(("(startFontClass)\n"));
   *m_t << "<emphasis role=\"" << colorClass << "\">";
   m_insideSpecialHL=TRUE;
@@ -213,6 +251,7 @@ void DocbookCodeGenerator::startFontClass(const QCString &colorClass)
 
 void DocbookCodeGenerator::endFontClass()
 {
+  if (m_hide) return;
   Docbook_DB(("(endFontClass)\n"));
   *m_t << "</emphasis>"; // non DocBook
   m_insideSpecialHL=FALSE;
@@ -226,6 +265,7 @@ void DocbookCodeGenerator::writeCodeAnchor(const QCString &)
 void DocbookCodeGenerator::writeLineNumber(const QCString &ref,const QCString &fileName,
     const QCString &anchor,int l,bool writeLineAnchor)
 {
+  if (m_hide) return;
   Docbook_DB(("(writeLineNumber)\n"));
   m_insideCodeLine = TRUE;
   if (Config_getBool(SOURCE_BROWSER))
@@ -268,8 +308,11 @@ DB_GEN_C1(*m_t)
 void DocbookCodeGenerator::endCodeFragment(const QCString &)
 {
 DB_GEN_C1(*m_t)
+  bool wasHidden = m_hide;
+  m_hide = false;
   //endCodeLine checks is there is still an open code line, if so closes it.
   endCodeLine();
+  m_hide = wasHidden;
 
   *m_t << "</programlisting>";
 }
@@ -336,7 +379,7 @@ void DocbookGenerator::init()
   Dir d(dir.str());
   if (!d.exists() && !d.mkdir(dir.str()))
   {
-    term("Could not create output directory %s\n",qPrint(dir));
+    term("Could not create output directory {}\n",dir);
   }
 
   createSubDirs(d);
@@ -349,7 +392,7 @@ void DocbookGenerator::cleanup()
 }
 
 
-void DocbookGenerator::startFile(const QCString &name,const QCString &,const QCString &,int,int)
+void DocbookGenerator::startFile(const QCString &name,bool,const QCString &,const QCString &,int,int)
 {
 DB_GEN_C
   QCString fileName=name;
@@ -574,7 +617,7 @@ DB_GEN_C2("IndexSection " << is)
         {
           if (dd->isLinkableInProject())
           {
-            m_t << "<    xi:include href=\"" << dd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            m_t << "    <xi:include href=\"" << dd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
           }
         }
       }
@@ -587,7 +630,7 @@ DB_GEN_C2("IndexSection " << is)
         {
           if (nd->isLinkableInProject() && !nd->isAlias())
           {
-            m_t << "<xi:include href=\"" << nd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            m_t << "    <xi:include href=\"" << nd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
           }
         }
       }
@@ -600,7 +643,7 @@ DB_GEN_C2("IndexSection " << is)
         {
           if (cd->isLinkableInProject() && !cd->isAlias())
           {
-            m_t << "<xi:include href=\"" << cd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            m_t << "    <xi:include href=\"" << cd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
           }
         }
       }
@@ -612,7 +655,7 @@ DB_GEN_C2("IndexSection " << is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==nullptr &&
+             !cd->isImplicitTemplateInstance() &&
              !cd->isEmbeddedInOuterScope() &&
              !cd->isAlias()
              )
@@ -683,11 +726,11 @@ DB_GEN_C
     m_pageLinks += link;
 }
 
-void DocbookGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int)
+void DocbookGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int,int sectionLevel)
 {
 DB_GEN_C
   auto astImpl = dynamic_cast<const DocNodeAST*>(ast);
-  if (astImpl)
+  if (astImpl && sectionLevel<=m_tocState.maxLevel)
   {
     DocbookDocVisitor visitor(m_t,*m_codeList,ctx?ctx->getDefFileExtension():QCString());
     std::visit(visitor,astImpl->root);
@@ -790,7 +833,7 @@ void DocbookGenerator::endBold()
 DB_GEN_C
   m_t << "</emphasis>";
 }
-void DocbookGenerator::startGroupHeader(int extraIndentLevel)
+void DocbookGenerator::startGroupHeader(const QCString &,int extraIndentLevel)
 {
 DB_GEN_C2("m_inLevel " << m_inLevel)
 DB_GEN_C2("extraIndentLevel " << extraIndentLevel)
@@ -929,7 +972,7 @@ void DocbookGenerator::endMemberDocName()
 {
 DB_GEN_C
 }
-void DocbookGenerator::startMemberGroupHeader(bool)
+void DocbookGenerator::startMemberGroupHeader(const QCString &,bool)
 {
 DB_GEN_C
   m_t << "<simplesect><title>";
@@ -1372,56 +1415,65 @@ DB_GEN_C
   m_t << theTranslator->trInheritedFrom(convertToDocBook(title), objectLinkToString(ref, file, anchor, name));
 }
 
-void DocbookGenerator::writeLocalToc(const SectionRefs &sectionRefs,const LocalToc &localToc)
+void DocbookGenerator::startLocalToc(int level)
 {
-  if (localToc.isDocbookEnabled())
+  m_tocState.level=1;
+  m_tocState.maxLevel=level;
+  m_tocState.inLi = BoolVector(level+1,false);
+  m_t << "    <toc>\n";
+  m_t << "    <title>" << theTranslator->trRTFTableOfContents() << "</title>\n";
+}
+
+void DocbookGenerator::endLocalToc()
+{
+  if (m_tocState.level > m_tocState.maxLevel) m_tocState.level = m_tocState.maxLevel;
+  while (m_tocState.level>1 && m_tocState.level <= m_tocState.maxLevel)
   {
-    m_t << "    <toc>\n";
-    m_t << "    <title>" << theTranslator->trRTFTableOfContents() << "</title>\n";
-    int level=1;
-    int maxLevel = localToc.docbookLevel();
-    BoolVector inLi(maxLevel+1,false);
-    for (const SectionInfo *si : sectionRefs)
+    m_t << "</tocdiv>\n";
+    m_tocState.level--;
+  }
+  m_t << "    </toc>\n";
+}
+
+void DocbookGenerator::startTocEntry(const SectionInfo *si)
+{
+  SectionType type = si->type();
+  if (type.isSection())
+  {
+    //printf("  level=%d title=%s\n",level,qPrint(si->title));
+    int nextLevel = type.level();
+    if (nextLevel>m_tocState.level)
     {
-      SectionType type = si->type();
-      if (type.isSection())
+      for (int l=m_tocState.level;l<nextLevel;l++)
       {
-        //printf("  level=%d title=%s\n",level,qPrint(si->title));
-        int nextLevel = type.level();
-        if (nextLevel>level)
-        {
-          for (int l=level;l<nextLevel;l++)
-          {
-            if (l < maxLevel) m_t << "    <tocdiv>\n";
-          }
-        }
-        else if (nextLevel<level)
-        {
-          for (int l=level;l>nextLevel;l--)
-          {
-            inLi[l]=FALSE;
-            if (l <= maxLevel) m_t << "    </tocdiv>\n";
-          }
-        }
-        if (nextLevel <= maxLevel)
-        {
-          QCString titleDoc = convertToDocBook(si->title());
-          QCString label    = convertToDocBook(si->label());
-          if (titleDoc.isEmpty()) titleDoc = label;
-          m_t << "      <tocentry>" << titleDoc << "</tocentry>\n";
-        }
-        inLi[nextLevel]=TRUE;
-        level = nextLevel;
+        if (l < m_tocState.maxLevel) m_t << "    <tocdiv>\n";
       }
     }
-    if (level > maxLevel) level = maxLevel;
-    while (level>1 && level <= maxLevel)
+    else if (nextLevel<m_tocState.level)
     {
-      inLi[level]=FALSE;
-      m_t << "</tocdiv>\n";
-      level--;
+      for (int l=m_tocState.level;l>nextLevel;l--)
+      {
+        m_tocState.inLi[l]=false;
+        if (l <= m_tocState.maxLevel) m_t << "    </tocdiv>\n";
+      }
     }
-    m_t << "    </toc>\n";
+    if (nextLevel <= m_tocState.maxLevel)
+    {
+      QCString label = convertToDocBook(si->label());
+      m_t << "      <tocentry>";
+    }
+  }
+}
+
+void DocbookGenerator::endTocEntry(const SectionInfo *si)
+{
+  SectionType type = si->type();
+  int nextLevel = type.level();
+  if (type.isSection() && nextLevel <= m_tocState.maxLevel)
+  {
+    m_t << "</tocentry>\n";
+    m_tocState.inLi[nextLevel]=true;
+    m_tocState.level = nextLevel;
   }
 }
 
