@@ -58,7 +58,6 @@ struct EclipseHelp::Private
 
 EclipseHelp::EclipseHelp() : p(std::make_unique<Private>()) {}
 EclipseHelp::~EclipseHelp() = default;
-EclipseHelp::EclipseHelp(EclipseHelp&&) = default;
 
 /*!
  * \brief Initialize the Eclipse generator
@@ -73,7 +72,7 @@ void EclipseHelp::initialize()
   p->tocstream = Portable::openOutputStream(name);
   if (!p->tocstream.is_open())
   {
-    term("Could not open file %s for writing\n", qPrint(name));
+    term("Could not open file {} for writing\n", name);
   }
 
   // -- write the opening tag
@@ -158,6 +157,7 @@ void EclipseHelp::decContentsDepth()
  * @param separateIndex not used.
  * @param addToNavIndex not used.
  * @param def not used.
+ * @param nameAsHtml name parameter in HTML format
  */
 void EclipseHelp::addContentsItem(
     bool /* isDir */,
@@ -167,12 +167,15 @@ void EclipseHelp::addContentsItem(
     const QCString &anchor,
     bool /* separateIndex */,
     bool /* addToNavIndex */,
-    const Definition * /*def*/)
+    const Definition * /*def*/,
+    const QCString & /* nameAsHtml */)
 {
   // -- write the topic tag
   p->closedTag();
   if (!file.isEmpty())
   {
+    QCString fn = file;
+    addHtmlExtensionIfMissing(fn);
     switch (file[0]) // check for special markers (user defined URLs)
     {
       case '^':
@@ -189,8 +192,7 @@ void EclipseHelp::addContentsItem(
       default:
         p->indent();
         p->tocstream << "<topic label=\"" << convertToXML(name) << "\"";
-        p->tocstream << " href=\"" << convertToXML(p->pathprefix)
-                    << addHtmlExtensionIfMissing(file);
+        p->tocstream << " href=\"" << convertToXML(p->pathprefix) << fn;
         if (!anchor.isEmpty())
         {
           p->tocstream << "#" << anchor;
