@@ -49,10 +49,12 @@
 #include "moduledef.h"
 
 static QCString g_header;
+static QCString g_header_file;
 static QCString g_footer;
+static QCString g_footer_file;
 static const SelectionMarkerInfo latexMarkerInfo = { '%', "%%BEGIN ",8 ,"%%END ",6, "",0 };
 
-static QCString substituteLatexKeywords(const QCString &str, const QCString &title);
+static QCString substituteLatexKeywords(const QCString &file, const QCString &str, const QCString &title);
 
 LatexCodeGenerator::LatexCodeGenerator(TextStream *t,const QCString &relPath,const QCString &sourceFileName)
   : m_t(t), m_relPath(relPath), m_sourceFileName(sourceFileName)
@@ -83,38 +85,53 @@ void LatexCodeGenerator::codify(const QCString &str)
     int tabSize = Config_getInt(TAB_SIZE);
     static THREAD_LOCAL char *result = nullptr;
     static THREAD_LOCAL int lresult = 0;
-    while ((c=*p))
+    if (m_hide) // only update column count
     {
-      switch(c)
+      m_col = updateColumnCount(p,m_col);
+    }
+    else // actually output content and keep track of m_col
+    {
+      while ((c=*p))
       {
-        case 0x0c: p++;  // remove ^L
-                   break;
-        case ' ':  *m_t << (m_doxyCodeLineOpen ? "\\ " : " ");
-                   m_col++;
-                   p++;
-                   break;
-        case '^':  *m_t <<"\\string^";
-                   m_col++;
-                   p++;
-                   break;
-        case '`':  *m_t <<"\\`{}";
-                   m_col++;
-                   p++;
-                   break;
-        case '\t': {
-                     int spacesToNextTabStop = tabSize - (m_col%tabSize);
-                     for (int i = 0; i < spacesToNextTabStop; i++) *m_t << (m_doxyCodeLineOpen ? "\\ " : " ");
-                     m_col+=spacesToNextTabStop;
+        switch(c)
+        {
+          case 0x0c: p++;  // remove ^L
+                     break;
+          case ' ':  if (m_col>=m_stripIndentAmount)
+                     {
+                       *m_t << (m_doxyCodeLineOpen ? "\\ " : " ");
+                     }
+                     m_col++;
                      p++;
-                   }
-                   break;
-        case '\n': *m_t << '\n';
-                   m_col=0;
-                   p++;
-                   break;
-        default:
-                   {
-                     int i=0;
+                     break;
+          case '^':  *m_t <<"\\string^";
+                     m_col++;
+                     p++;
+                     break;
+          case '`':  *m_t <<"\\`{}";
+                     m_col++;
+                     p++;
+                     break;
+          case '\t': {
+                       int spacesToNextTabStop = tabSize - (m_col%tabSize);
+                       while (spacesToNextTabStop--)
+                       {
+                         if (m_col>=m_stripIndentAmount)
+                         {
+                           *m_t << (m_doxyCodeLineOpen ? "\\ " : " ");
+                         }
+                         m_col++;
+                       }
+                       p++;
+                     }
+                     break;
+          case '\n': *m_t << '\n';
+                     m_col=0;
+                     p++;
+                     break;
+          default:
+                     {
+                       int i=0;
 
 #undef  COPYCHAR
 // helper macro to copy a single utf8 character, dealing with multibyte chars.
@@ -132,35 +149,56 @@ void LatexCodeGenerator::codify(const QCString &str)
                        m_col++;                                     \
                      } while(0)
 
-                     // gather characters until we find whitespace or another special character
-                     COPYCHAR();
-                     while ((c=*p) &&
-                            c!=0x0c && c!='\t' && c!='\n' && c!=' ' && c!='^'
-                           )
-                     {
+                       // gather characters until we find whitespace or another special character
                        COPYCHAR();
+                       while ((c=*p) &&
+                              c!=0x0c && c!='\t' && c!='\n' && c!=' ' && c!='^'
+                             )
+                       {
+                         COPYCHAR();
+                       }
+                       result[i]=0; // add terminator
+                       filterLatexString(*m_t,result,
+                                         m_insideTabbing, // insideTabbing
+                                         true,  // insidePre
+                                         false, // insideItem
+                                         m_usedTableLevel>0, // insideTable
+                                         false  // keepSpaces
+                                        );
                      }
-                     result[i]=0; // add terminator
-                     filterLatexString(*m_t,result,
-                                       m_insideTabbing, // insideTabbing
-                                       true,  // insidePre
-                                       false, // insideItem
-                                       m_usedTableLevel>0, // insideTable
-                                       false  // keepSpaces
-                                      );
-                   }
-                   break;
+                     break;
+        }
       }
     }
   }
 }
 
+void LatexCodeGenerator::stripCodeComments(bool b)
+{
+  m_stripCodeComments = b;
+}
+
+void LatexCodeGenerator::startSpecialComment()
+{
+  m_hide = m_stripCodeComments;
+}
+
+void LatexCodeGenerator::endSpecialComment()
+{
+  m_hide = false;
+}
+
+void LatexCodeGenerator::setStripIndentAmount(size_t amount)
+{
+  m_stripIndentAmount = amount;
+}
 
 void LatexCodeGenerator::writeCodeLink(CodeSymbolType,
                                    const QCString &ref,const QCString &f,
                                    const QCString &anchor,const QCString &name,
                                    const QCString &)
 {
+  if (m_hide) return;
   bool pdfHyperlinks = Config_getBool(PDF_HYPERLINKS);
   bool usePDFLatex   = Config_getBool(USE_PDFLATEX);
   size_t l = name.length();
@@ -183,6 +221,7 @@ void LatexCodeGenerator::writeCodeLink(CodeSymbolType,
 
 void LatexCodeGenerator::writeLineNumber(const QCString &ref,const QCString &fileName,const QCString &anchor,int l,bool writeLineAnchor)
 {
+  if (m_hide) return;
   bool usePDFLatex = Config_getBool(USE_PDFLATEX);
   bool pdfHyperlinks = Config_getBool(PDF_HYPERLINKS);
   if (!m_doxyCodeLineOpen)
@@ -229,6 +268,7 @@ void LatexCodeGenerator::writeLineNumber(const QCString &ref,const QCString &fil
 
 void LatexCodeGenerator::startCodeLine(int)
 {
+  if (m_hide) return;
   m_col=0;
   if (!m_doxyCodeLineOpen)
   {
@@ -239,6 +279,7 @@ void LatexCodeGenerator::startCodeLine(int)
 
 void LatexCodeGenerator::endCodeLine()
 {
+  if (m_hide) return;
   if (m_doxyCodeLineOpen)
   {
     *m_t << "}";
@@ -249,11 +290,13 @@ void LatexCodeGenerator::endCodeLine()
 
 void LatexCodeGenerator::startFontClass(const QCString &name)
 {
+  if (m_hide) return;
   *m_t << "\\textcolor{" << name << "}{";
 }
 
 void LatexCodeGenerator::endFontClass()
 {
+  if (m_hide) return;
   *m_t << "}";
 }
 
@@ -265,7 +308,10 @@ void LatexCodeGenerator::startCodeFragment(const QCString &style)
 void LatexCodeGenerator::endCodeFragment(const QCString &style)
 {
   //endCodeLine checks is there is still an open code line, if so closes it.
+  bool wasHidden = m_hide;
+  m_hide = false;
   endCodeLine();
+  m_hide = wasHidden;
 
   *m_t << "\\end{" << style << "}\n";
 }
@@ -325,7 +371,7 @@ static void writeLatexMakefile()
   std::ofstream f = Portable::openOutputStream(fileName);
   if (!f.is_open())
   {
-    term("Could not open file %s for writing\n",qPrint(fileName));
+    term("Could not open file {} for writing\n",fileName);
   }
   TextStream t(&f);
   // inserted by KONNO Akihisa <konno@researchers.jp> 2002-03-05
@@ -468,7 +514,7 @@ static void writeMakeBat()
   std::ofstream t = Portable::openOutputStream(fileName);
   if (!t.is_open())
   {
-    term("Could not open file %s for writing\n",qPrint(fileName));
+    term("Could not open file {} for writing\n",fileName);
   }
   t << "pushd %~dp0\r\n";
   t << "if not %errorlevel% == 0 goto :end1\r\n";
@@ -590,33 +636,37 @@ void LatexGenerator::init()
   Dir d(dname.str());
   if (!d.exists() && !d.mkdir(dname.str()))
   {
-    term("Could not create output directory %s\n",qPrint(dname));
+    term("Could not create output directory {}\n",dname);
   }
 
   if (!Config_getString(LATEX_HEADER).isEmpty())
   {
-    g_header=fileToString(Config_getString(LATEX_HEADER));
+    g_header_file=Config_getString(LATEX_HEADER);
+    g_header=fileToString(g_header_file);
     //printf("g_header='%s'\n",qPrint(g_header));
-    QCString result = substituteLatexKeywords(g_header,QCString());
-    checkBlocks(result,Config_getString(LATEX_HEADER),latexMarkerInfo);
+    QCString result = substituteLatexKeywords(g_header_file,g_header,QCString());
+    checkBlocks(result,g_header_file,latexMarkerInfo);
   }
   else
   {
-    g_header = ResourceMgr::instance().getAsString("header.tex");
-    QCString result = substituteLatexKeywords(g_header,QCString());
+    g_header_file = "header.tex";
+    g_header = ResourceMgr::instance().getAsString(g_header_file);
+    QCString result = substituteLatexKeywords(g_header_file,g_header,QCString());
     checkBlocks(result,"<default header.tex>",latexMarkerInfo);
   }
   if (!Config_getString(LATEX_FOOTER).isEmpty())
   {
-    g_footer=fileToString(Config_getString(LATEX_FOOTER));
+    g_footer_file=Config_getString(LATEX_FOOTER);
+    g_footer=fileToString(g_footer_file);
     //printf("g_footer='%s'\n",qPrint(g_footer));
-    QCString result = substituteLatexKeywords(g_footer,QCString());
-    checkBlocks(result,Config_getString(LATEX_FOOTER),latexMarkerInfo);
+    QCString result = substituteLatexKeywords(g_footer_file,g_footer,QCString());
+    checkBlocks(result,g_footer_file,latexMarkerInfo);
   }
   else
   {
-    g_footer = ResourceMgr::instance().getAsString("footer.tex");
-    QCString result = substituteLatexKeywords(g_footer,QCString());
+    g_footer_file = "footer.tex";
+    g_footer = ResourceMgr::instance().getAsString(g_footer_file);
+    QCString result = substituteLatexKeywords(g_footer_file,g_footer,QCString());
     checkBlocks(result,"<default footer.tex>",latexMarkerInfo);
   }
 
@@ -656,7 +706,7 @@ void LatexGenerator::writeStyleSheetFile(TextStream &t)
   writeDefaultStyleSheet(t);
 }
 
-void LatexGenerator::startFile(const QCString &name,const QCString &,const QCString &,int,int hierarchyLevel)
+void LatexGenerator::startFile(const QCString &name,bool,const QCString &,const QCString &,int,int hierarchyLevel)
 {
 #if 0
   setEncoding(Config_getString(LATEX_OUTPUT_ENCODING));
@@ -698,14 +748,15 @@ static QCString extraLatexStyleSheet()
       if (fi.exists())
       {
         result += "\\usepackage{";
-        if (checkExtension(fi.fileName().c_str(), LATEX_STYLE_EXTENSION))
+        QCString fn = fi.fileName();
+        if (checkExtension(fn, LATEX_STYLE_EXTENSION))
         {
           // strip the extension, it will be added by the usepackage in the tex conversion process
-          result += stripExtensionGeneral(fi.fileName().c_str(), LATEX_STYLE_EXTENSION);
+          result += stripExtensionGeneral(fn, LATEX_STYLE_EXTENSION);
         }
         else
         {
-          result += fi.fileName();
+          result += fn;
         }
         result += "}\n";
       }
@@ -746,7 +797,8 @@ static QCString latex_batchmode()
   return "";
 }
 
-static QCString substituteLatexKeywords(const QCString &str,
+static QCString substituteLatexKeywords(const QCString &file,
+                                        const QCString &str,
                                         const QCString &title)
 {
   bool compactLatex = Config_getBool(COMPACT_LATEX);
@@ -757,7 +809,7 @@ static QCString substituteLatexKeywords(const QCString &str,
   QCString style = Config_getString(LATEX_BIB_STYLE);
   if (style.isEmpty())
   {
-    style="plain";
+    style="plainnat";
   }
 
   TextStream tg;
@@ -814,15 +866,19 @@ static QCString substituteLatexKeywords(const QCString &str,
   QCString projectNumber = Config_getString(PROJECT_NUMBER);
 
   // first substitute generic keywords
-  QCString result = substituteKeywords(str,title,
+  QCString result = substituteKeywords(file,str,title,
         convertToLaTeX(Config_getString(PROJECT_NAME),false),
         convertToLaTeX(projectNumber,false),
         convertToLaTeX(Config_getString(PROJECT_BRIEF),false));
 
   // additional LaTeX only keywords
-  result = substituteKeywords(result,
+  result = substituteKeywords(file,result,
   {
     // keyword                     value getter
+    { "$datetime",                 [&]() { return dateToString(DateTimeType::DateTime);         } },
+    { "$date",                     [&]() { return dateToString(DateTimeType::Date);             } },
+    { "$time",                     [&]() { return dateToString(DateTimeType::Time);             } },
+    { "$year",                     [&]() { return yearToString();                               } },
     { "$latexdocumentpre",         [&]() { return theTranslator->latexDocumentPre();            } },
     { "$latexdocumentpost",        [&]() { return theTranslator->latexDocumentPost();           } },
     { "$generatedby",              [&]() { return generatedBy;                                  } },
@@ -867,7 +923,7 @@ void LatexGenerator::startIndexSection(IndexSection is)
   switch (is)
   {
     case IndexSection::isTitlePageStart:
-      m_t << substituteLatexKeywords(g_header,convertToLaTeX(Config_getString(PROJECT_NAME),m_codeGen->insideTabbing()));
+      m_t << substituteLatexKeywords(g_header_file,g_header,convertToLaTeX(Config_getString(PROJECT_NAME),m_codeGen->insideTabbing()));
       break;
     case IndexSection::isTitlePageAuthor:
       break;
@@ -979,7 +1035,7 @@ void LatexGenerator::startIndexSection(IndexSection is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==nullptr &&
+              !cd->isImplicitTemplateInstance() &&
               !cd->isEmbeddedInOuterScope() &&
               !cd->isAlias()
              )
@@ -1151,7 +1207,7 @@ void LatexGenerator::endIndexSection(IndexSection is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==nullptr &&
+             !cd->isImplicitTemplateInstance() &&
              !cd->isEmbeddedInOuterScope() &&
              !cd->isAlias()
              )
@@ -1219,7 +1275,7 @@ void LatexGenerator::endIndexSection(IndexSection is)
     case IndexSection::isPageDocumentation2:
       break;
     case IndexSection::isEndIndex:
-      m_t << substituteLatexKeywords(g_footer,convertToLaTeX(Config_getString(PROJECT_NAME),m_codeGen->insideTabbing()));
+      m_t << substituteLatexKeywords(g_footer_file,g_footer,convertToLaTeX(Config_getString(PROJECT_NAME),m_codeGen->insideTabbing()));
       break;
   }
 }
@@ -1242,14 +1298,6 @@ void LatexGenerator::writeStyleInfo(int part)
   writeDefaultStyleSheet(m_t);
   endPlainFile();
 
-  // workaround for the problem caused by change in LaTeX in version 2019
-  // in the unmaintained tabu package
-  startPlainFile("tabu_doxygen.sty");
-  m_t << ResourceMgr::instance().getAsString("tabu_doxygen.sty");
-  endPlainFile();
-  startPlainFile("longtable_doxygen.sty");
-  m_t << ResourceMgr::instance().getAsString("longtable_doxygen.sty");
-  endPlainFile();
   /// an extension of the etoc package is required that is only available in the
   /// newer version. Providing the updated version to be used with older versions
   /// of LaTeX
@@ -1452,19 +1500,7 @@ void LatexGenerator::endTitleHead(const QCString &fileName,const QCString &name)
   }
 }
 
-void LatexGenerator::startTitle()
-{
-  if (Config_getBool(COMPACT_LATEX))
-  {
-    m_t << "\\doxysubsection{";
-  }
-  else
-  {
-    m_t << "\\doxysection{";
-  }
-}
-
-void LatexGenerator::startGroupHeader(int extraIndentLevel)
+void LatexGenerator::startGroupHeader(const QCString &,int extraIndentLevel)
 {
   if (Config_getBool(COMPACT_LATEX))
   {
@@ -1930,7 +1966,7 @@ void LatexGenerator::endMemberList()
 }
 
 
-void LatexGenerator::startMemberGroupHeader(bool hasHeader)
+void LatexGenerator::startMemberGroupHeader(const QCString &,bool hasHeader)
 {
   if (hasHeader) m_t << "\\begin{Indent}";
   m_t << "\\textbf{ ";
@@ -2090,7 +2126,7 @@ void LatexGenerator::exceptionEntry(const QCString &prefix,bool closeBracket)
   m_t << " ";
 }
 
-void LatexGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int)
+void LatexGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int,int)
 {
   const DocNodeAST *astImpl = dynamic_cast<const DocNodeAST*>(ast);
   if (astImpl)
@@ -2267,14 +2303,11 @@ void LatexGenerator::writeInheritedSectionTitle(
   m_t << "}\n";
 }
 
-void LatexGenerator::writeLocalToc(const SectionRefs &,const LocalToc &localToc)
+void LatexGenerator::startLocalToc(int level)
 {
-  if (localToc.isLatexEnabled())
-  {
-    int maxLevel = localToc.latexLevel() + m_hierarchyLevel;
-    m_t << "\\etocsetnexttocdepth{" << maxLevel << "}\n";
-    m_t << "\\localtableofcontents\n";
-  }
+  int maxLevel = level + m_hierarchyLevel;
+  m_t << "\\etocsetnexttocdepth{" << maxLevel << "}\n";
+  m_t << "\\localtableofcontents\n";
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -2289,9 +2322,9 @@ void writeExtraLatexPackages(TextStream &t)
     for (const auto &pkgName : extraPackages)
     {
       if ((pkgName[0] == '[') || (pkgName[0] == '{'))
-        t << "\\usepackage" << pkgName.c_str() << "\n";
+        t << "\\usepackage" << pkgName << "\n";
       else
-        t << "\\usepackage{" << pkgName.c_str() << "}\n";
+        t << "\\usepackage{" << pkgName << "}\n";
     }
     t << "\n";
   }
@@ -2471,7 +2504,15 @@ void filterLatexString(TextStream &t,const QCString &str,
                    break;
         case '\'': t << "\\textquotesingle{}";
                    break;
-        case '\n':  if (retainNewline) t << "\\newline"; else t << ' ';
+        case '\n': if (retainNewline)
+                   {
+                     t << "\\newline";
+                     if (insideTable) t << " ";
+                   }
+                   else
+                   {
+                     t << ' ';
+                   }
                    break;
         case ' ':  if (keepSpaces) { if (insideTabbing) t << "\\>"; else t << '~'; } else t << ' ';
                    break;
