@@ -1,11 +1,17 @@
 #!/usr/bin/env python
 
 from __future__ import print_function
-import argparse, glob, itertools, re, shutil, os, sys
+import argparse
+import glob
+import itertools
+import re
+import shutil
+import os
+import sys
 import subprocess
 import shlex
 
-config_reg = re.compile('.*\/\/\s*(?P<name>\S+):\s*(?P<value>.*)$')
+config_reg = re.compile(r'.*\/\/\s*(?P<name>\S+):\s*(?P<value>.*)$')
 bkmk_reg = re.compile(r'.*bkmkstart\s+([A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z]).*')
 hyper_reg = re.compile(r'.*HYPERLINK\s+[\\l]*\s+"([A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z])".*')
 pageref_reg = re.compile(r'.*PAGEREF\s+([A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z][A-Z]).*')
@@ -65,6 +71,9 @@ class Tester:
         self.test      = test
         self.update    = args.updateref
         self.config    = self.get_config()
+        if 'objective' not in self.config:
+            print("Test %s is missing the objective." % self.test)
+            sys.exit(1)
         self.test_name = '[%s]: %s' % (self.test,self.config['objective'][0])
         self.test_id   = self.test.split('_')[0]
         if self.update:
@@ -165,6 +174,12 @@ class Tester:
                 print('DOCBOOK_OUTPUT=%s/docbook' % self.test_out, file=f)
             else:
                 print('GENERATE_DOCBOOK=NO', file=f)
+            if (self.args.man):
+                print('GENERATE_MAN=YES', file=f)
+                print('MAN_LINKS=YES', file=f)
+                print('MAN_OUTPUT=%s/man' % self.test_out, file=f)
+            else:
+                print('GENERATE_MAN=NO', file=f)
             if (self.args.qhp):
                 print('GENERATE_QHP=YES', file=f)
             if (self.args.xhtml or self.args.qhp):
@@ -200,7 +215,7 @@ class Tester:
         if (self.args.noredir):
             redir=''
 
-        if os.system('%s %s/Doxyfile %s' % (self.args.doxygen,self.test_out,redir))!=0:
+        if os.system('%s %s %s/Doxyfile %s' % (self.args.doxygen,self.args.doxygen_dbg,self.test_out,redir))!=0:
             print('Error: failed to run %s on %s/Doxyfile' % (self.args.doxygen,self.test_out))
             sys.exit(1)
 
@@ -265,20 +280,16 @@ class Tester:
                     print(data,file=f)
         shutil.rmtree(self.test_out+'/out',ignore_errors=True)
         os.remove(self.test_out+'/Doxyfile')
+        os.remove(self.test_out+'/warnings.log')
         return True
 
     # check the relevant files of a doxygen run with the reference material
     def perform_test(self,testmgr):
         if (sys.platform == 'win32'):
-            redir=' > nul:'
             separ='&'
         else:
-            redir=' 2> /dev/null'
             separ=';'
 
-        if (self.args.noredir):
-            redir=''
-
         failed_xml=False
         failed_html=False
         failed_qhp=False
@@ -286,6 +297,7 @@ class Tester:
         failed_docbook=False
         failed_rtf=False
         failed_xmlxsd=False
+        failed_man=False
         msg = ()
         # look for files to check against the reference
         if self.args.xml or self.args.xmlxsd:
@@ -450,6 +462,10 @@ class Tester:
             elif not self.args.keep:
                 shutil.rmtree(docbook_output,ignore_errors=True)
 
+        if (self.args.man):
+            # For future work
+            pass
+
         if (self.args.xhtml or self.args.qhp):
             html_output='%s/html' % self.test_out
             if (sys.platform == 'win32'):
@@ -507,7 +523,7 @@ class Tester:
             if outType:
                 exe_string += ' %s more temp' % (separ)
             latex_out = xpopen(exe_string,exe_string1,getStderr=outType)
-            os.chdir(cur_directory);
+            os.chdir(cur_directory)
             if (outType and latex_out.find("Error")!=-1):
                 msg += ("PDF generation failed\n  For a description of the problem see 'refman.log' in the latex directory of this test",)
                 failed_latex=True
@@ -528,7 +544,7 @@ class Tester:
         if failed_warn:
             msg += (warnings,)
 
-        if failed_warn or failed_xml or failed_html or failed_qhp or failed_latex or failed_docbook or failed_rtf or failed_xmlxsd:
+        if failed_warn or failed_xml or failed_html or failed_qhp or failed_latex or failed_docbook or failed_rtf or failed_xmlxsd or failed_man:
             testmgr.ok(False,self.test_name,msg)
             return False
 
@@ -619,6 +635,8 @@ def main():
         'update the reference file(s) for the given test',action="store_true")
     parser.add_argument('--doxygen',nargs='?',default='doxygen',help=
         'path/name of the doxygen executable')
+    parser.add_argument('--doxygen_dbg',nargs='?',default='',help=
+        'the doxygen debugging arguments')
     parser.add_argument('--xmllint',nargs='?',default='xmllint',help=
         'path/name of the xmllint executable')
     parser.add_argument('--id',nargs='+',dest='ids',action='append',type=int,help=
@@ -652,6 +670,8 @@ def main():
         'create qhp output and check with xmllint',action="store_true")
     parser.add_argument('--xmlxsd',help=
         'create xml output and check with xmllint against xsd',action="store_true")
+    parser.add_argument('--man',help=
+        'create man output',action="store_true")
     parser.add_argument('--pdf',help='create LaTeX output and create pdf from it',
         action="store_true")
     parser.add_argument('--subdirs',help='use the configuration parameter CREATE_SUBDIRS=YES',
@@ -670,9 +690,9 @@ def main():
     args = parser.parse_args(test_flags + sys.argv[1:])
 
     # sanity check
-    if (not args.xml) and (not args.pdf) and (not args.xhtml) and (not args.qhp) and (not args.docbook and (not args.rtf) and (not args.xmlxsd)):
+    if (not args.xml) and (not args.pdf) and (not args.xhtml) and (not args.qhp) and (not args.docbook and (not args.rtf) and (not args.xmlxsd) and (not args.man)):
         args.xml=True
-    if (not args.updateref is None) and (args.ids is None) and (args.all is None):
+    if (args.updateref is not None) and (args.ids is None) and (args.all is None):
         parser.error('--updateref requires either --id or --all')
 
     starting_directory = os.getcwd()
